theme_colors = {
    "background_primary":"#F4F4F8",

    "background_secondary": "#ECEBF0",
    
    "background_tertiary": "#959598",

    "interactive_primary": "#3B82F6",
    "interactive_hover": "#60A5FA",
    "interactive_pressed": "#2563EB",
    "interactive_text": "#FFFFFF",

    # Texto Principal (Preto quase absoluto)
    "text_primary": "#000000",

    "text_secondary": "#000000",
    
    "text_disabled": "#9CA3AF",

    "success": "#059669",
    "warning": "#D97706",
    "warning_hover": "#F59E0B",
    "danger": "#DC2626",
    "info": "#0284C7",
}


import re
from PyQt5.QtWidgets import (
    QWidget, QVBoxLayout, QHBoxLayout, QLineEdit, QLabel, 
    QListView, QStyledItemDelegate, QStyle, QApplication
)
from PyQt5.QtCore import Qt, QSize, QAbstractListModel, QModelIndex, QRect, QRectF
from PyQt5.QtGui import QColor, QPainter, QFont, QPen, QFontMetrics, QPainterPath

from gui.widgets.agent_detail_window import AgentDetailWindow
from gui.assets.theme.styler import current_theme
def natural_sort_key(s):
    return [int(text) if text.isdigit() else text.lower()
            for text in re.split(r'(\d+)', s)]

class AgentListModel(QAbstractListModel):
    def __init__(self, agents_list=None, parent=None):
        super().__init__(parent)
        self._agents = agents_list or []

    def rowCount(self, parent=QModelIndex()):
        return len(self._agents)

    def data(self, index, role=Qt.DisplayRole):
        if not index.isValid():
            return None
        if role == Qt.DisplayRole:
            return self._agents[index.row()]
        return None

    def update_list(self, new_list):
        self.beginResetModel()
        self._agents = new_list
        self.endResetModel()

class AgentVisualDelegate(QStyledItemDelegate):
    def __init__(self, data_model, parent_view, parent=None):
        super().__init__(parent)
        self.data_model = data_model
        self.parent_view = parent_view
        self._update_colors()

    def _update_colors(self):

        self.c_bg = QColor(current_theme.get('background_secondary', '#1F2937'))
        self.c_border = QColor(current_theme.get('background_tertiary', '#4B5563'))
        self.c_border_hover = QColor(current_theme.get('interactive_primary', '#3B82F6'))
        self.c_text_pri = QColor(current_theme.get('text_primary', '#F9FAFB'))
        self.c_text_sec = QColor(current_theme.get('text_secondary', '#9CA3AF'))
        
        self.c_btn_bg = QColor(current_theme.get('interactive_primary', '#3B82F6'))
        self.c_btn_txt = QColor(current_theme.get('interactive_text', '#FFFFFF'))
        self.c_accent = QColor(current_theme.get('info', '#0EA5E9'))

    def update_theme(self):
        self._update_colors()

    def _extract_action_from_log(self, log_data):
        desc = getattr(log_data, "desc", "")
        if desc:
            match = re.search(r"doing action \*(\w+)", desc)
            if match: return f"Running: {match.group(1)}"
            match = re.search(r"action:\s*\*(\w+)", desc)
            if match: return f"Running: {match.group(1)}"
        
        running_intentions = getattr(log_data, "running_intentions", [])
        if not running_intentions: return "Idle"
        try:
            main_intention_str = running_intentions[0]
            match = re.search(r"->\s*([\w\d_]+)\(.*\)\s*Context=", main_intention_str)
            if match: return f"Plano: {match.group(1)}"
            match = re.search(r"\]\s*,\s*([\w\d_]+)\(.*\)", main_intention_str)
            if match: return f"Plano: {match.group(1)}"
        except (IndexError, TypeError): pass
        return "Running Intention"

    def paint(self, painter, option, index):
        agent_name = index.data(Qt.DisplayRole)
        state = self.data_model.get_latest_agent_state(agent_name)
        
        painter.save()
        painter.setRenderHint(QPainter.Antialiasing)

        rect = option.rect.adjusted(5, 5, -5, -5)
        is_hover = option.state & QStyle.State_MouseOver
        
        path = QPainterPath()
        path.addRoundedRect(QRectF(rect), 8, 8) 
        
        painter.setBrush(self.c_bg)
        if is_hover:
            painter.setPen(QPen(self.c_border_hover, 2))
        else:
            painter.setPen(QPen(self.c_border, 2))
        
        painter.drawPath(path)

        f_title = QFont(option.font); f_title.setBold(True); f_title.setPointSize(12)
        f_meta = QFont(option.font); f_meta.setPointSize(10)
        f_stat = QFont(option.font); f_stat.setBold(True); f_stat.setPointSize(11)
        f_btn = QFont(option.font); f_btn.setBold(True); f_btn.setPointSize(9)

        padding = 15
        current_y = rect.top() + padding

        fm_title = QFontMetrics(f_title)
        title_rect = QRect(rect.left() + padding, current_y, rect.width() - (2*padding), fm_title.height())
        
        painter.setFont(f_title)
        painter.setPen(self.c_text_pri)
        painter.drawText(title_rect, Qt.AlignLeft | Qt.AlignVCenter, agent_name)
        
        cycle_val = str(getattr(state, 'cycle', 0)) if state else "0"
        painter.setFont(f_meta)
        painter.setPen(self.c_text_sec)
        painter.drawText(title_rect, Qt.AlignRight | Qt.AlignVCenter, f"Cycle: {cycle_val}")

        current_y += title_rect.height() + 8

        beliefs_list = getattr(state, 'beliefs', []) if state else []
        goals_list = getattr(state, 'goals', []) if state else []
        
        painter.setFont(f_meta)
        painter.setPen(self.c_text_sec)
        
        stats_rect = QRect(rect.left() + padding, current_y, rect.width() - (2*padding), 20)
        painter.drawText(stats_rect, Qt.AlignLeft, f"Beliefs: {len(beliefs_list)}")
        painter.drawText(stats_rect.adjusted(100, 0, 0, 0), Qt.AlignLeft, f"Goals: {len(goals_list)}")

        current_y += 25

        action_str = self._extract_action_from_log(state) if state else "Aguardando..."
        action_rect = QRect(rect.left() + padding, current_y, rect.width() - (2*padding), 40)
        painter.setFont(f_stat)
        painter.setPen(self.c_accent)
        painter.drawText(action_rect, Qt.AlignLeft | Qt.AlignTop | Qt.TextWordWrap, action_str)

        btn_height = 30
        btn_rect = QRect(rect.left() + padding, rect.bottom() - padding - btn_height + 5, 
                         rect.width() - (2*padding), btn_height)
        
        btn_path = QPainterPath()
        btn_path.addRoundedRect(QRectF(btn_rect), 6, 6)
        
        painter.setBrush(self.c_btn_bg)
        painter.setPen(Qt.NoPen)
        painter.drawPath(btn_path)

        painter.setPen(self.c_btn_txt)
        painter.setFont(f_btn)
        painter.drawText(btn_rect, Qt.AlignCenter, "View Details")

        painter.restore()

    def sizeHint(self, option, index):
        if not self.parent_view or self.parent_view.viewport().width() <= 0:
            return QSize(250, 170)
        
        view_width = self.parent_view.viewport().width()
        available_width = view_width - 30
        
        cols = max(1, available_width // 260)
        card_width = int(available_width / cols) - 10
        
        if card_width < 220: card_width = 220
        
        return QSize(card_width, 170)

class AgentesPage(QWidget):
    def __init__(self, log_store, data_model):
        super().__init__()
        self.setProperty("class", "page")
        self.data_model = data_model
        self.log_store = log_store
        self.all_agents_list = []
        self.open_windows = {}

        self._setup_ui()

    def _setup_ui(self):
        layout = QVBoxLayout(self)
        layout.setContentsMargins(20, 20, 20, 20)
        layout.setSpacing(15)

        top_layout = QHBoxLayout()
        self.search_input = QLineEdit()
        self.search_input.setPlaceholderText("Search agents by name")
        self.search_input.setFixedHeight(40)
        self.search_input.textChanged.connect(self.on_search_changed)
        
        self.status_label = QLabel("Awaiting data...")
        self.status_label.setProperty("class", "text-secondary")
        
        top_layout.addWidget(self.search_input)
        top_layout.addWidget(self.status_label)
        top_layout.addStretch()
        layout.addLayout(top_layout)

        self.agent_view = QListView()
        self.agent_view.setViewMode(QListView.IconMode) 
        self.agent_view.setResizeMode(QListView.Adjust) 
        self.agent_view.setUniformItemSizes(True)
        self.agent_view.setSpacing(10)
        
        self.agent_view.setStyleSheet("""
            QListView { 
                background: transparent; 
                border: none; 
                outline: none;
            }
            QListView::item {
                background: transparent;
            }
            QListView::item:selected {
                background: transparent;
                border: none;
            }
        """)
        
        self.agent_view.clicked.connect(self.on_agent_clicked)
        
        self.model = AgentListModel()
        self.delegate = AgentVisualDelegate(self.data_model, self.agent_view)
        
        self.agent_view.setModel(self.model)
        self.agent_view.setItemDelegate(self.delegate)
        
        layout.addWidget(self.agent_view)

    def resizeEvent(self, event):
        super().resizeEvent(event)
        self.agent_view.scheduleDelayedItemsLayout()

    def on_agent_list_updated(self, agents):
        self.all_agents_list = sorted(agents, key=natural_sort_key)
        self._update_filter()

    def on_search_changed(self):
        self._update_filter()

    def _update_filter(self):
        txt = self.search_input.text().strip().lower()
        
        if not txt: 
            final_list = self.all_agents_list
        else: 
            final_list = [a for a in self.all_agents_list if txt in a.lower()]
            
        self.model.update_list(final_list)
        self.status_label.setText(f"Total: {len(final_list)} agents")

    def on_store_updated(self):
        self.agent_view.viewport().update()

    def on_agent_clicked(self, index):
        agent_name = index.data(Qt.DisplayRole)
        if not agent_name: 
            return

        if agent_name in self.open_windows:
            try:
                if self.open_windows[agent_name].isVisible():
                    self.open_windows[agent_name].activateWindow()
                    return
            except RuntimeError:
                pass

        detail_window = AgentDetailWindow(agent_name, self.data_model)
        detail_window.destroyed.connect(lambda: self.on_window_closed(agent_name))
        self.open_windows[agent_name] = detail_window
        detail_window.show()

    def on_window_closed(self, agent_name):
        self.data_model.stop_observing_agent(agent_name)
        if agent_name in self.open_windows:
            del self.open_windows[agent_name]

    def update_theme(self):

        self.delegate.update_theme()

        self.agent_view.viewport().update()
        for agent_name, window in list(self.open_windows.items()):
            try:
                if window.isVisible() and hasattr(window, 'update_theme'):
                    window.update_theme()
            except RuntimeError:
                pass


import re
import ast
from PyQt5.QtWidgets import (
    QWidget, QVBoxLayout, QLabel, QFrame,
    QHBoxLayout, QTreeWidget, QTreeWidgetItem, QHeaderView, 
    QListWidget, QListWidgetItem, QTabWidget, QSizePolicy,
    QListView, QAbstractItemView, QTableWidget, QTableWidgetItem,
    QStackedWidget, QComboBox, QGridLayout
)
from PyQt5.QtCore import Qt, QRectF
from PyQt5.QtGui import QColor, QFont, QBrush, QPainter, QPen, QPainterPath

from gui.assets.theme.styler import current_theme

from gui.core.history_model import HistoryLogModel, HistoryLogDelegate, HistoryItem, HistoryListView
from gui.widgets.percepts_table import PerceptsTableWidget
from gui.widgets.help_button import HelpButton


class StatusCard(QFrame):
    def __init__(self, parent=None):
        super().__init__(parent)
        self._data = {
            'cycle': '-',
            'action': '-',
            'event': '-',
            'intentions': '-'
        }
        
        self.setMinimumHeight(120)
        self.setMaximumHeight(140)
        self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
    
    def set_data(self, cycle, action, event, intentions):
        self._data = {
            'cycle': str(cycle),
            'action': str(action) if action else '-',
            'event': str(event) if event else '-',
            'intentions': str(intentions)
        }
        self.update()
    
    def paintEvent(self, event):
        painter = QPainter(self)
        painter.setRenderHint(QPainter.Antialiasing)
        
        rect = self.rect()
        
        bg_color = QColor(current_theme.get('background_secondary', '#1F2937'))
        border_color = QColor(current_theme.get('background_tertiary', '#4B5563'))
        text_primary = QColor(current_theme.get('text_primary', '#F9FAFB'))
        text_secondary = QColor(current_theme.get('text_secondary', '#9CA3AF'))
        accent = QColor(current_theme.get('interactive_primary', '#3B82F6'))
        
        bg_rect = QRectF(rect).adjusted(1, 1, -1, -1)
        path = QPainterPath()
        path.addRoundedRect(bg_rect, 10, 10)
        
        painter.setPen(QPen(border_color, 1))
        painter.setBrush(QBrush(bg_color))
        painter.drawPath(path)
        
        accent_rect = QRectF(bg_rect.left() + 4, bg_rect.top() + 12, 4, bg_rect.height() - 24)
        accent_path = QPainterPath()
        accent_path.addRoundedRect(accent_rect, 2, 2)
        painter.setPen(Qt.NoPen)
        painter.setBrush(QBrush(accent))
        painter.drawPath(accent_path)
        
        content_left = bg_rect.left() + 20
        content_width = bg_rect.width() - 32
        col_width = content_width / 2
        row_height = (bg_rect.height() - 24) / 2
        
        label_font = QFont("Segoe UI", 9)
        value_font = QFont("Segoe UI", 11, QFont.Bold)
        
        items = [
            ('Cycle', self._data['cycle'], 0, 0),
            ('Intentions', self._data['intentions'], 0, 1),
            ('Current Action', self._data['action'], 1, 0),
            ('Processing Event', self._data['event'], 1, 1),
        ]
        
        for label, value, row, col in items:
            x = content_left + col * col_width
            y = bg_rect.top() + 12 + row * row_height
            
            painter.setFont(label_font)
            painter.setPen(text_secondary)
            painter.drawText(QRectF(x, y, col_width - 10, 16), 
                           Qt.AlignLeft | Qt.AlignVCenter, label)
            
            painter.setFont(value_font)
            painter.setPen(text_primary)
            fm = painter.fontMetrics()
            truncated = fm.elidedText(str(value), Qt.ElideRight, int(col_width - 20))
            painter.drawText(QRectF(x, y + 18, col_width - 10, 24), 
                           Qt.AlignLeft | Qt.AlignVCenter, truncated)
        
        painter.end()


class MiniInfoCard(QFrame):
    def __init__(self, title, accent_color=None, parent=None):
        super().__init__(parent)
        self._title = title
        self._items = []
        self._accent_color = accent_color or "#3B82F6"
        
        self.setMinimumHeight(80)
        self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
    
    def set_items(self, items):
        self._items = items if items else []
        self.update()
    
    def paintEvent(self, event):
        painter = QPainter(self)
        painter.setRenderHint(QPainter.Antialiasing)
        
        rect = self.rect()
        
        bg_color = QColor(current_theme.get('background_secondary', '#1F2937'))
        border_color = QColor(current_theme.get('background_tertiary', '#4B5563'))
        text_primary = QColor(current_theme.get('text_primary', '#F9FAFB'))
        text_secondary = QColor(current_theme.get('text_secondary', '#9CA3AF'))
        accent = QColor(self._accent_color)
        
        bg_rect = QRectF(rect).adjusted(1, 1, -1, -1)
        path = QPainterPath()
        path.addRoundedRect(bg_rect, 8, 8)
        
        painter.setPen(QPen(border_color, 1))
        painter.setBrush(QBrush(bg_color))
        painter.drawPath(path)
        
        accent_rect = QRectF(bg_rect.left() + 4, bg_rect.top() + 8, 3, bg_rect.height() - 16)
        accent_path = QPainterPath()
        accent_path.addRoundedRect(accent_rect, 1.5, 1.5)
        painter.setPen(Qt.NoPen)
        painter.setBrush(QBrush(accent))
        painter.drawPath(accent_path)
        
        content_left = bg_rect.left() + 16
        
        title_font = QFont("Segoe UI", 9)
        painter.setFont(title_font)
        painter.setPen(text_secondary)
        count_text = f"{self._title} ({len(self._items)})"
        painter.drawText(QRectF(content_left, bg_rect.top() + 8, bg_rect.width() - 24, 18), 
                        Qt.AlignLeft | Qt.AlignVCenter, count_text)
        
        value_font = QFont("Segoe UI", 10)
        painter.setFont(value_font)
        painter.setPen(text_primary)
        
        if self._items:
            items_text = ", ".join(self._items[:5])
            if len(self._items) > 5:
                items_text += f" (+{len(self._items) - 5})"
        else:
            items_text = "None"
            painter.setPen(text_secondary)
        
        fm = painter.fontMetrics()
        truncated = fm.elidedText(items_text, Qt.ElideRight, int(bg_rect.width() - 30))
        painter.drawText(QRectF(content_left, bg_rect.top() + 32, bg_rect.width() - 24, 24), 
                        Qt.AlignLeft | Qt.AlignVCenter, truncated)
        
        painter.end()


class LearningInfoCard(QFrame):
    def __init__(self, parent=None):
        super().__init__(parent)
        self._environment = "-"
        self._status = "-"
        self._status_color = "#9CA3AF"
        
        self.setMinimumHeight(80)
        self.setMaximumHeight(90)
        self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
    
    def set_data(self, environment, status, is_complete):
        self._environment = environment
        self._status = status
        
        if is_complete:
            self._status_color = current_theme.get('success', '#10B981')
        else:
            self._status_color = current_theme.get('warning', '#F59E0B')
        
        self.update()
    
    def clear(self):
        self._environment = "-"
        self._status = "-"
        self._status_color = "#9CA3AF"
        self.update()
    
    def paintEvent(self, event):
        painter = QPainter(self)
        painter.setRenderHint(QPainter.Antialiasing)
        
        rect = self.rect()
        
        bg_color = QColor(current_theme.get('background_secondary', '#1F2937'))
        border_color = QColor(current_theme.get('background_tertiary', '#4B5563'))
        text_primary = QColor(current_theme.get('text_primary', '#F9FAFB'))
        text_secondary = QColor(current_theme.get('text_secondary', '#9CA3AF'))
        accent = QColor(current_theme.get('interactive_primary', '#3B82F6'))
        
        bg_rect = QRectF(rect).adjusted(1, 1, -1, -1)
        path = QPainterPath()
        path.addRoundedRect(bg_rect, 8, 8)
        
        painter.setPen(QPen(border_color, 1))
        painter.setBrush(QBrush(bg_color))
        painter.drawPath(path)
        
        accent_rect = QRectF(bg_rect.left() + 4, bg_rect.top() + 10, 4, bg_rect.height() - 20)
        accent_path = QPainterPath()
        accent_path.addRoundedRect(accent_rect, 2, 2)
        painter.setPen(Qt.NoPen)
        painter.setBrush(QBrush(accent))
        painter.drawPath(accent_path)
        
        content_left = bg_rect.left() + 18
        content_width = bg_rect.width() - 28
        col_width = content_width / 2
        
        label_font = QFont("Segoe UI", 9)
        value_font = QFont("Segoe UI", 11)
        
        y_top = bg_rect.top() + 16
        
        painter.setFont(label_font)
        painter.setPen(text_secondary)
        painter.drawText(QRectF(content_left, y_top, col_width, 16), 
                        Qt.AlignLeft | Qt.AlignVCenter, "Environment")
        
        painter.setFont(value_font)
        painter.setPen(text_primary)
        painter.drawText(QRectF(content_left, y_top + 20, col_width - 10, 24), 
                        Qt.AlignLeft | Qt.AlignVCenter, self._environment)
        
        col2_left = content_left + col_width
        painter.setFont(label_font)
        painter.setPen(text_secondary)
        painter.drawText(QRectF(col2_left, y_top, col_width, 16), 
                        Qt.AlignLeft | Qt.AlignVCenter, "Training Status")
        
        painter.setFont(value_font)
        painter.setPen(QColor(self._status_color))
        painter.drawText(QRectF(col2_left, y_top + 20, col_width, 24), 
                        Qt.AlignLeft | Qt.AlignVCenter, self._status)
        
        painter.end()


class ModelSelectorCard(QFrame):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.setMinimumHeight(70)
        self.setMaximumHeight(80)
        self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
        self._setup_ui()
    
    def _setup_ui(self):
        layout = QHBoxLayout(self)
        layout.setContentsMargins(16, 12, 16, 12)
        layout.setSpacing(16)
        
        label = QLabel("Model")
        label.setFont(QFont("Segoe UI", 10))
        label.setProperty("class", "text-secondary")
        
        self.combo = QComboBox()
        self.combo.setMinimumWidth(250)
        self.combo.setMinimumHeight(36)
        self.combo.setCursor(Qt.PointingHandCursor)
        
        layout.addWidget(label)
        layout.addWidget(self.combo)
        layout.addStretch()
    
    def style_combo(self):
        bg_secondary = current_theme.get('background_secondary', '#1F2937')
        bg_tertiary = current_theme.get('background_tertiary', '#4B5563')
        text_primary = current_theme.get('text_primary', '#F9FAFB')
        interactive = current_theme.get('interactive_primary', '#3B82F6')
        
        self.combo.setStyleSheet(f"""
            QComboBox {{
                background-color: {bg_secondary};
                border: 1px solid {bg_tertiary};
                border-radius: 6px;
                padding: 8px 12px;
                color: {text_primary};
                font-size: 12px;
            }}
            QComboBox:hover {{
                border: 1px solid {interactive};
            }}
            QComboBox::drop-down {{
                border: none;
                padding-right: 10px;
            }}
            QComboBox QAbstractItemView {{
                background-color: {bg_secondary};
                border: 1px solid {bg_tertiary};
                selection-background-color: {interactive};
                color: {text_primary};
            }}
        """)
    
    def paintEvent(self, event):
        painter = QPainter(self)
        painter.setRenderHint(QPainter.Antialiasing)
        
        rect = self.rect()
        
        bg_color = QColor(current_theme.get('background_secondary', '#1F2937'))
        border_color = QColor(current_theme.get('background_tertiary', '#4B5563'))
        accent = QColor(current_theme.get('interactive_primary', '#3B82F6'))
        
        bg_rect = QRectF(rect).adjusted(1, 1, -1, -1)
        path = QPainterPath()
        path.addRoundedRect(bg_rect, 8, 8)
        
        painter.setPen(QPen(border_color, 1))
        painter.setBrush(QBrush(bg_color))
        painter.drawPath(path)
        
        accent_rect = QRectF(bg_rect.left() + 4, bg_rect.top() + 8, 3, bg_rect.height() - 16)
        accent_path = QPainterPath()
        accent_path.addRoundedRect(accent_rect, 1.5, 1.5)
        painter.setPen(Qt.NoPen)
        painter.setBrush(QBrush(accent))
        painter.drawPath(accent_path)
        
        painter.end()
        
        super().paintEvent(event)


class AgentDetailWindow(QWidget):
    def __init__(self, agent_name, data_model, parent=None):
        super().__init__(parent)
        self.agent_name = agent_name
        self.data_model = data_model
        self.setObjectName("AgentDetailWindow")
        
        self.setWindowTitle(f"Agent's Details: {self.agent_name}")
        self.setGeometry(150, 150, 900, 750) 
        self.setAttribute(Qt.WA_DeleteOnClose) 

        self.last_log_index = 0
        self.last_intention_in_history = None
        self.regex_change = re.compile(r"(Adding|Removing) Info: (Belief|Goal) (.*?)\s+-\s+instant\[.*\]$")

        self.belief_items = []
        self.goal_items = []
        self.intention_items = []

        self.previous_beliefs_set = set()
        self.previous_goals_set = set()
        self.active_beliefs = set()
        self.active_goals = set()
        self.successful_goals = set()
        self.changes_this_cycle = set()

        self.belief_model = HistoryLogModel()
        self.goal_model = HistoryLogModel()
        self.intention_model = HistoryLogModel()

        main_layout = QVBoxLayout(self)
        main_layout.setContentsMargins(15, 15, 15, 15)
        main_layout.setSpacing(10)
        
        title = QLabel(f"Monitoring: {self.agent_name}")
        title.setFont(QFont("Segoe UI", 18, QFont.Bold))
        title.setAlignment(Qt.AlignCenter)
        main_layout.addWidget(title)
        
        tab_widget = QTabWidget()
        main_layout.addWidget(tab_widget)

        tab_widget.addTab(self._create_overview_tab(), "Overview")
        tab_widget.addTab(self._create_mind_tab(), "Agent's Mind")
        tab_widget.addTab(self._create_perception_tab(), "Environment and Channels")
        tab_widget.addTab(self._create_learning_tab(), "Learning")
        
        self.data_model.agent_data_updated.connect(self.on_agent_data_updated)

        self.update_data()

    def update_theme(self):
        for view in [self.belief_view, self.goal_view, self.intention_view]:
            delegate = view.itemDelegate()
            if hasattr(delegate, 'update_theme_colors'):
                delegate.update_theme_colors()
            view.viewport().update()

        if hasattr(self, 'perceptions_table'):
            self.perceptions_table.update_theme()
        
        if hasattr(self, 'status_card'):
            self.status_card.update()
        if hasattr(self, 'env_card'):
            self.env_card.update()
        if hasattr(self, 'ch_card'):
            self.ch_card.update()
        if hasattr(self, 'learning_info_card'):
            self.learning_info_card.update()
        if hasattr(self, 'model_selector_card'):
            self.model_selector_card.style_combo()
            self.model_selector_card.update()
        
        self.repaint()

    def on_agent_data_updated(self, updated_agent_name):
        if updated_agent_name == self.agent_name:
            self.update_data()

    def update_data(self):
        full_history = self.data_model.get_agent_log_history(self.agent_name)
        if not full_history: return
        
        latest_state = full_history[-1]
        self._update_snapshots(latest_state)

        self._update_learning_tab()

        total_logs = len(full_history)
        if total_logs == self.last_log_index: return

        new_logs = full_history[self.last_log_index:]
        self.last_log_index = total_logs
        self._process_new_logs_to_models(new_logs, full_history)

    def _normalize_belief_or_goal(self, item):
        if isinstance(item, dict):
            tipo = item.get('type', '')
            predicate = item.get('predicate', '')
            args = item.get('args', [])
            source = item.get('source', '')
            args_str = ", ".join(map(str, args))
            return f"{tipo}: {predicate}({args_str}) [{source}]"
        return str(item)

    def _get_beliefs_set(self, log):
        beliefs = getattr(log, 'beliefs', []) or []
        return {self._normalize_belief_or_goal(b) for b in beliefs}

    def _get_goals_set(self, log):
        goals = getattr(log, 'goals', []) or []
        return {self._normalize_belief_or_goal(g) for g in goals}

    def _process_new_logs_to_models(self, new_logs, full_history):
        items_added = False
        
        for i, log in enumerate(new_logs):
            desc = log.desc
            timestamp = log.system_time
            cycle = log.cycle
            
            current_beliefs = self._get_beliefs_set(log)
            current_goals = self._get_goals_set(log)
            
            def record_change(item_type, msg_type, content):
                change_key = (cycle, item_type, msg_type, content)
                if change_key in self.changes_this_cycle:
                    return False
                self.changes_this_cycle.add(change_key)
                return True
            
            gained_beliefs = current_beliefs - self.previous_beliefs_set
            for belief_str in gained_beliefs:
                if belief_str not in self.active_beliefs:
                    if record_change('belief', 'GAIN', belief_str):
                        item = HistoryItem('GAIN', belief_str, timestamp, cycle)
                        self.belief_items.append(item)
                        self.active_beliefs.add(belief_str)
                        items_added = True
            
            lost_beliefs = self.previous_beliefs_set - current_beliefs
            for belief_str in lost_beliefs:
                if belief_str in self.active_beliefs:
                    if record_change('belief', 'LOSE', belief_str):
                        item = HistoryItem('LOSE', belief_str, timestamp, cycle)
                        self.belief_items.append(item)
                        self.active_beliefs.discard(belief_str)
                        items_added = True
            
            gained_goals = current_goals - self.previous_goals_set
            for goal_str in gained_goals:
                if goal_str not in self.active_goals:
                    if record_change('goal', 'GAIN', goal_str):
                        item = HistoryItem('GAIN', goal_str, timestamp, cycle)
                        self.goal_items.append(item)
                        self.active_goals.add(goal_str)
                        items_added = True
            
            lost_goals = self.previous_goals_set - current_goals
            for goal_str in lost_goals:
                if goal_str in self.active_goals:
                    if record_change('goal', 'LOSE', goal_str):
                        item = HistoryItem('LOSE', goal_str, timestamp, cycle)
                        self.goal_items.append(item)
                        self.active_goals.discard(goal_str)
                        items_added = True
            
            match = self.regex_change.match(desc)
            if not match:
                match = self.regex_change.search(desc)
            
            if match:
                action_type = match.group(1)
                item_type = match.group(2)
                content = match.group(3).strip()
                msg_type = 'GAIN' if action_type == "Adding" else 'LOSE'
                normalized_content = f"{item_type} {content}"
                
                if item_type == 'Belief':
                    if msg_type == 'GAIN' and normalized_content not in self.active_beliefs:
                        if record_change('belief', 'GAIN', normalized_content):
                            item = HistoryItem(msg_type, normalized_content, timestamp, cycle)
                            self.belief_items.append(item)
                            self.active_beliefs.add(normalized_content)
                            items_added = True
                    elif msg_type == 'LOSE' and normalized_content in self.active_beliefs:
                        if record_change('belief', 'LOSE', normalized_content):
                            item = HistoryItem(msg_type, normalized_content, timestamp, cycle)
                            self.belief_items.append(item)
                            self.active_beliefs.discard(normalized_content)
                            items_added = True
                elif item_type == 'Goal':
                    if msg_type == 'GAIN' and normalized_content not in self.active_goals:
                        if record_change('goal', 'GAIN', normalized_content):
                            item = HistoryItem(msg_type, normalized_content, timestamp, cycle)
                            self.goal_items.append(item)
                            self.active_goals.add(normalized_content)
                            items_added = True
                    elif msg_type == 'LOSE' and normalized_content in self.active_goals:
                        if record_change('goal', 'LOSE', normalized_content):
                            item = HistoryItem(msg_type, normalized_content, timestamp, cycle)
                            self.goal_items.append(item)
                            self.active_goals.discard(normalized_content)
                            items_added = True
            
            self.previous_beliefs_set = current_beliefs
            self.previous_goals_set = current_goals
            
            for evt in log.events:
                if "success:" in evt and "Goal" in evt:
                    content = evt.replace("success:", "")
                    if content not in self.successful_goals:
                        item = HistoryItem('SUCCESS', content, timestamp, cycle)
                        self.goal_items.append(item)
                        self.successful_goals.add(content)
                        items_added = True

            curr = log.last_intention
            if curr and curr != 'null' and curr != self.last_intention_in_history:
                event_str = log.last_event.replace('gain:', '').replace('lose:', '') if log.last_event else ''
                content_str = f"Trigger: {event_str} | {curr}"
                item = HistoryItem('INTENTION', content_str, timestamp, cycle)
                self.intention_items.append(item)
                self.last_intention_in_history = curr
                items_added = True

        if items_added:
            self.belief_model.set_items(self.belief_items)
            self.goal_model.set_items(self.goal_items)
            self.intention_model.set_items(self.intention_items)
            self._scroll_to_bottom(self.belief_view, self.belief_model)
            self._scroll_to_bottom(self.goal_view, self.goal_model)
            self._scroll_to_bottom(self.intention_view, self.intention_model)
            
            self._update_mind_counters()

    def _scroll_to_bottom(self, view, model):
        if model.rowCount() > 0:
            view.scrollTo(model.index(model.rowCount() - 1, 0), QAbstractItemView.EnsureVisible)

    def _update_snapshots(self, latest_state):
        self.status_card.set_data(
            latest_state.cycle,
            latest_state.action,
            latest_state.last_event,
            len(latest_state.running_intentions)
        )
        
        self._update_list_widget(self.running_intentions_list, latest_state.running_intentions)
        self._update_list_widget(self.current_beliefs_list, latest_state.beliefs)
        self._update_list_widget(self.current_goals_list, latest_state.goals)
        
        self.perceptions_table.set_percepts(latest_state.perceptions)
        
        self.env_card.set_items(latest_state.envs)
        self.ch_card.set_items(latest_state.chs)

    def _create_section_title(self, text):
        title = QLabel(text)
        title.setProperty("class", "h2")
        title.setFont(QFont("Segoe UI", 12, QFont.Bold))
        return title
    
    def _create_section_title_with_counter(self, text):
        container = QWidget()
        layout = QHBoxLayout(container)
        layout.setContentsMargins(0, 0, 0, 0)
        layout.setSpacing(8)
        
        title = QLabel(text)
        title.setProperty("class", "h2")
        title.setFont(QFont("Segoe UI", 12, QFont.Bold))
        
        counter = QLabel("(0)")
        counter.setProperty("class", "text-secondary")
        counter.setFont(QFont("Segoe UI", 11))
        
        layout.addWidget(title)
        layout.addWidget(counter)
        layout.addStretch()
        
        return container, counter
        
    def _create_frame(self):
        frame = QFrame()
        frame.setProperty("class", "card")
        return frame

    def _create_key_value_display(self, parent_layout, key_text):
        layout = QHBoxLayout()
        key_label = QLabel(f"<b>{key_text}</b>")
        key_label.setProperty("class", "detail-key")
        value_label = QLabel("<i>N/A</i>")
        value_label.setWordWrap(True)
        value_label.setProperty("class", "detail-value")
        value_label.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Preferred)
        value_label.setMinimumWidth(100)
        layout.addWidget(key_label)
        layout.addStretch()
        layout.addWidget(value_label)
        parent_layout.addLayout(layout)
        return value_label

    def _create_list_display(self, min_height=120):
        list_widget = QListWidget()
        list_widget.setMinimumHeight(min_height)
        list_widget.setFrameShape(QFrame.NoFrame)
        list_widget.setWordWrap(True)
        list_widget.setHorizontalScrollBarPolicy(Qt.ScrollBarAsNeeded)
        list_widget.setHorizontalScrollMode(QAbstractItemView.ScrollPerPixel)
        return list_widget

    def _create_log_list_view(self, model, min_height=150):
        view = HistoryListView()
        view.setModel(model)
        
        delegate = HistoryLogDelegate(view, current_theme)
        view.setItemDelegate(delegate)
        
        view.setStyleSheet("QListView { font-size: 13pt; font-family: Arial; }")
        view.setUniformItemSizes(False)
        view.setWordWrap(False)
        view.setAlternatingRowColors(False)
        view.setSelectionMode(QAbstractItemView.ExtendedSelection)
        view.setMinimumHeight(min_height)
        view.setVerticalScrollMode(QAbstractItemView.ScrollPerPixel)
        view.setHorizontalScrollBarPolicy(Qt.ScrollBarAsNeeded)
        view.setHorizontalScrollMode(QAbstractItemView.ScrollPerPixel)
        view.setFrameShape(QFrame.NoFrame)
        
        return view
        
    def _create_tree_display(self, headers):
        tree = QTreeWidget()
        tree.setHeaderLabels(headers)
        tree.header().setSectionResizeMode(0, QHeaderView.ResizeToContents)
        if len(headers) > 1:
            tree.header().setSectionResizeMode(1, QHeaderView.ResizeToContents)
        tree.header().setStretchLastSection(False)
        tree.setHorizontalScrollBarPolicy(Qt.ScrollBarAsNeeded)
        tree.setVerticalScrollBarPolicy(Qt.ScrollBarAsNeeded)
        tree.setHorizontalScrollMode(QAbstractItemView.ScrollPerPixel)
        return tree

    def _create_overview_tab(self):
        tab = QWidget()
        layout = QHBoxLayout(tab) 
        layout.setContentsMargins(15, 15, 15, 15)
        layout.setSpacing(15)

        left_col_layout = QVBoxLayout()
        left_col_layout.setSpacing(12)
        
        left_col_layout.addWidget(self._create_section_title("Real Time Status"))
        
        self.status_card = StatusCard()
        left_col_layout.addWidget(self.status_card)
        
        left_col_layout.addWidget(self._create_section_title("Running Intentions"))
        
        intentions_frame = self._create_frame()
        intentions_layout = QVBoxLayout(intentions_frame)
        intentions_layout.setContentsMargins(12, 12, 12, 12)
        self.running_intentions_list = self._create_list_display(min_height=150)
        intentions_layout.addWidget(self.running_intentions_list)
        left_col_layout.addWidget(intentions_frame)
        
        left_col_layout.addStretch()
        layout.addLayout(left_col_layout)

        right_col_layout = QVBoxLayout()
        right_col_layout.setSpacing(12)
        
        right_col_layout.addWidget(self._create_section_title("Current Beliefs"))
        beliefs_frame = self._create_frame()
        beliefs_layout = QVBoxLayout(beliefs_frame)
        beliefs_layout.setContentsMargins(12, 12, 12, 12)
        self.current_beliefs_list = self._create_list_display()
        beliefs_layout.addWidget(self.current_beliefs_list)
        right_col_layout.addWidget(beliefs_frame)
        
        right_col_layout.addWidget(self._create_section_title("Current Goals"))
        goals_frame = self._create_frame()
        goals_layout = QVBoxLayout(goals_frame)
        goals_layout.setContentsMargins(12, 12, 12, 12)
        self.current_goals_list = self._create_list_display()
        goals_layout.addWidget(self.current_goals_list)
        right_col_layout.addWidget(goals_frame)
        
        right_col_layout.addStretch()
        layout.addLayout(right_col_layout)

        return tab

    def _create_mind_tab(self):
        tab = QWidget()
        main_layout = QVBoxLayout(tab) 
        main_layout.setSpacing(15)
        main_layout.setContentsMargins(15, 15, 15, 15)

        top_layout = QHBoxLayout()
        top_layout.setSpacing(15)
        
        belief_layout = QVBoxLayout()
        belief_header, self.belief_counter = self._create_section_title_with_counter("Beliefs History")
        belief_layout.addWidget(belief_header)
        
        b_frame = self._create_frame()
        b_frame_layout = QVBoxLayout(b_frame)
        b_frame_layout.setContentsMargins(8, 8, 8, 8)
        
        self.belief_view = self._create_log_list_view(self.belief_model)
        b_frame_layout.addWidget(self.belief_view)
        
        belief_layout.addWidget(b_frame)
        top_layout.addLayout(belief_layout)

        goal_layout = QVBoxLayout()
        goal_header, self.goal_counter = self._create_section_title_with_counter("Goals History")
        goal_layout.addWidget(goal_header)
        
        g_frame = self._create_frame()
        g_frame_layout = QVBoxLayout(g_frame)
        g_frame_layout.setContentsMargins(8, 8, 8, 8)
        
        self.goal_view = self._create_log_list_view(self.goal_model)
        g_frame_layout.addWidget(self.goal_view)
        
        goal_layout.addWidget(g_frame)
        top_layout.addLayout(goal_layout)
        
        main_layout.addLayout(top_layout, stretch=1) 

        int_layout = QVBoxLayout()
        int_header, self.intention_counter = self._create_section_title_with_counter("Intentions History")
        int_layout.addWidget(int_header)
        
        i_frame = self._create_frame()
        i_frame_layout = QVBoxLayout(i_frame)
        i_frame_layout.setContentsMargins(8, 8, 8, 8)
        
        self.intention_view = self._create_log_list_view(self.intention_model)
        i_frame_layout.addWidget(self.intention_view)
        
        int_layout.addWidget(i_frame)
        main_layout.addLayout(int_layout, stretch=1) 

        return tab
    
    def _update_mind_counters(self):
        self.belief_counter.setText(f"({len(self.belief_items)})")
        self.goal_counter.setText(f"({len(self.goal_items)})")
        self.intention_counter.setText(f"({len(self.intention_items)})")

    def _create_perception_tab(self):
        tab = QWidget()
        main_layout = QHBoxLayout(tab) 
        main_layout.setSpacing(15)
        main_layout.setContentsMargins(15, 15, 15, 15)

        percept_frame = self._create_frame()
        percept_layout = QVBoxLayout(percept_frame)
        percept_layout.setContentsMargins(16, 16, 16, 16)
        
        percept_title = QLabel("Perceptions")
        percept_title.setFont(QFont("Segoe UI", 12, QFont.Bold))
        percept_layout.addWidget(percept_title)
        
        self.perceptions_table = PerceptsTableWidget()
        percept_layout.addWidget(self.perceptions_table)
        
        main_layout.addWidget(percept_frame, stretch=2) 

        right_frame = self._create_frame()
        right_layout = QVBoxLayout(right_frame)
        right_layout.setContentsMargins(16, 16, 16, 16)
        right_layout.setSpacing(12)
        
        right_title = QLabel("Connections")
        right_title.setFont(QFont("Segoe UI", 12, QFont.Bold))
        right_layout.addWidget(right_title)
        
        self.env_card = MiniInfoCard("Environments", "#10B981")  # green
        self.ch_card = MiniInfoCard("Channels", "#8B5CF6")  # purple
        
        right_layout.addWidget(self.env_card)
        right_layout.addWidget(self.ch_card)
        right_layout.addStretch()
        
        main_layout.addWidget(right_frame, stretch=1) 
        
        return tab

    def _parse_intention_list_for_display(self, intention_str):
        if not isinstance(intention_str, str): return str(intention_str)
        try:
            goal_match = re.search(r"^(gain:Goal.*?\[.*?\])", intention_str)
            plan_match = re.search(r"->\s*([\w\d_]+)\(.*\)", intention_str)
            goal_str = goal_match.group(1) if goal_match else "Complex Goal"
            plan_name = plan_match.group(1) if plan_match else "Complex Plan"
            return f"Plan: {plan_name} | Goal: {goal_str}"
        except Exception: return intention_str 
            
    def _update_list_widget(self, list_widget, items_list):
        list_widget.clear()
        if not items_list:
            item = QListWidgetItem("No items")
            item.setForeground(QColor(current_theme.get('text_disabled', '#888')))
            list_widget.addItem(item)
        else:
            for item_data in items_list:
                display_text = ""
                if list_widget == self.running_intentions_list:
                    display_text = self._parse_intention_list_for_display(item_data)
                elif isinstance(item_data, dict):
                    display_text = self._format_parsed_object_for_display(item_data)
                else:
                    display_text = str(item_data)
                list_widget.addItem(QListWidgetItem(display_text))

    def _format_parsed_object_for_display(self, parsed_item):
        if not isinstance(parsed_item, dict): return str(parsed_item)
        if not parsed_item or parsed_item.get('type') == 'Unknown': return parsed_item.get('raw', str(parsed_item))
        tipo = parsed_item.get('type', '')
        predicate = parsed_item.get('predicate', '')
        args = parsed_item.get('args', [])
        source = parsed_item.get('source', '')
        args_str = ", ".join(map(str, args))
        return f"{tipo}: {predicate}({args_str}) [{source}]"

    def _safe_eval(self, literal):
        try: return ast.literal_eval(literal)
        except (ValueError, SyntaxError): return literal

    def _populate_tree(self, tree, data, parent_item=None):
        if parent_item is None:
            tree.clear()
            parent_item = tree.invisibleRootItem()
        if isinstance(data, str): data = self._safe_eval(data)
        if isinstance(data, dict):
            for key, val in sorted(data.items()):
                key_str = str(self._safe_eval(key))
                child = QTreeWidgetItem([key_str])
                parent_item.addChild(child)
                self._populate_tree(tree, val, child)
        elif isinstance(data, (list, set, tuple)):
            for i, val in enumerate(data):
                child = QTreeWidgetItem([f"Item {i}"])
                parent_item.addChild(child)
                self._populate_tree(tree, val, child)
        else:
            if parent_item.childCount() == 0 and parent_item.text(0) != "root":
                parent_item.setText(1, str(data))
        if parent_item == tree.invisibleRootItem():
            tree.expandAll()
    
    def _create_learning_tab(self):
        tab = QWidget()
        layout = QVBoxLayout(tab)
        layout.setContentsMargins(15, 15, 15, 15)
        layout.setSpacing(12)

        self.learning_status_label = QLabel("No learning models found for this agent")
        self.learning_status_label.setProperty("class", "text-secondary")
        self.learning_status_label.setFont(QFont("Segoe UI", 11))
        layout.addWidget(self.learning_status_label)

        self.model_selector_card = ModelSelectorCard()
        self.model_selector_card.combo.currentTextChanged.connect(self._on_learning_model_selected)
        self.model_selector_card.style_combo()
        layout.addWidget(self.model_selector_card)

        self.learning_info_card = LearningInfoCard()
        layout.addWidget(self.learning_info_card)

        qtable_frame = self._create_frame()
        qtable_layout = QVBoxLayout(qtable_frame)
        qtable_layout.setContentsMargins(16, 16, 16, 16)
        
        qtable_label = QLabel("Q-Table (Policy)")
        qtable_label.setFont(QFont("Segoe UI", 12, QFont.Bold))
        qtable_layout.addWidget(qtable_label)
        
        self.learning_qtable = QTableWidget()
        self.learning_qtable.setObjectName("LearningQTable")
        self.learning_qtable.setAlternatingRowColors(False)
        self.learning_qtable.setSelectionBehavior(QAbstractItemView.SelectRows)
        self.learning_qtable.setEditTriggers(QAbstractItemView.NoEditTriggers)
        self.learning_qtable.horizontalHeader().setStretchLastSection(True)
        self.learning_qtable.verticalHeader().setVisible(False)
        self.learning_qtable.setFrameShape(QFrame.NoFrame)
        self._style_table_header(self.learning_qtable)
        qtable_layout.addWidget(self.learning_qtable, 1)
        
        layout.addWidget(qtable_frame, 1)

        self._learning_data = {}
        
        return tab
    
    def _style_table_header(self, table):
        bg_tertiary = current_theme.get('background_tertiary', '#4B5563')
        text_primary = current_theme.get('text_primary', '#F9FAFB')
        interactive = current_theme.get('interactive_primary', '#3B82F6')

        table.horizontalHeader().setStyleSheet(f"""
            QHeaderView::section {{
                background-color: {bg_tertiary};
                color: {text_primary};
                padding: 8px;
                border: none;
                border-bottom: 2px solid {interactive};
                font-weight: bold;
            }}
        """)
    
    def _on_learning_model_selected(self, model_name):
        if not model_name or model_name not in self._learning_data:
            return
        
        model_data = self._learning_data[model_name]
        environment = model_data.get('environment', '-')
        is_complete = model_data.get('training_complete', False)
        status = "Training Complete" if is_complete else "In Progress"
        
        self.learning_info_card.set_data(environment, status, is_complete)
        self._update_learning_qtable(model_data.get('policy', {}))
    
    def _update_learning_qtable(self, policy):
        self.learning_qtable.clear()
        
        if not policy:
            self.learning_qtable.setRowCount(0)
            self.learning_qtable.setColumnCount(1)
            self.learning_qtable.setHorizontalHeaderLabels(["No data"])
            return

        first_state = next(iter(policy.values()))
        num_actions = len(first_state) if isinstance(first_state, list) else 1

        columns = ["State"] + [f"Action {i}" for i in range(num_actions)]
        self.learning_qtable.setColumnCount(len(columns))
        self.learning_qtable.setHorizontalHeaderLabels(columns)

        row_color_1 = QColor(current_theme.get('background_secondary', '#1F2937'))
        row_color_2 = QColor(current_theme.get('background_tertiary', '#4B5563'))
        text_color = QColor(current_theme.get('text_primary', '#F9FAFB'))

        best_value_color = QColor(current_theme.get('success', '#10B981'))
        best_value_color.setAlpha(100)

        self.learning_qtable.setRowCount(len(policy))
        
        for row, (state, values) in enumerate(sorted(policy.items())):
            row_bg = row_color_1 if row % 2 == 0 else row_color_2

            state_item = QTableWidgetItem(str(state))
            state_item.setTextAlignment(Qt.AlignCenter)
            state_item.setBackground(QBrush(row_bg))
            state_item.setForeground(QBrush(text_color))
            self.learning_qtable.setItem(row, 0, state_item)

            if isinstance(values, list):
                max_val = max(values)
                for col, val in enumerate(values):
                    val_item = QTableWidgetItem(f"{val:.4f}")
                    val_item.setTextAlignment(Qt.AlignCenter)
                    val_item.setForeground(QBrush(text_color))

                    if val == max_val:
                        val_item.setBackground(QBrush(best_value_color))
                        val_item.setFont(QFont("Segoe UI", 10, QFont.Bold))
                    else:
                        val_item.setBackground(QBrush(row_bg))
                    
                    self.learning_qtable.setItem(row, col + 1, val_item)
            else:
                val_item = QTableWidgetItem(str(values))
                val_item.setTextAlignment(Qt.AlignCenter)
                val_item.setBackground(QBrush(row_bg))
                val_item.setForeground(QBrush(text_color))
                self.learning_qtable.setItem(row, 1, val_item)

        self.learning_qtable.horizontalHeader().setSectionResizeMode(0, QHeaderView.ResizeToContents)
        for i in range(1, len(columns)):
            self.learning_qtable.horizontalHeader().setSectionResizeMode(i, QHeaderView.Stretch)

        self._style_table_header(self.learning_qtable)
    
    def _update_learning_tab(self):
        self._learning_data = self.data_model.get_learning_for_agent(self.agent_name)
        
        if not self._learning_data:
            self.learning_status_label.setText("No learning models found for this agent")
            self.learning_status_label.setVisible(True)
            self.model_selector_card.combo.clear()
            self.learning_info_card.clear()
            self.learning_qtable.clear()
            self.learning_qtable.setRowCount(0)
            return
        
        self.learning_status_label.setVisible(False)

        current_model = self.model_selector_card.combo.currentText()
        self.model_selector_card.combo.blockSignals(True)
        self.model_selector_card.combo.clear()
        self.model_selector_card.combo.addItems(sorted(self._learning_data.keys()))

        if current_model in self._learning_data:
            self.model_selector_card.combo.setCurrentText(current_model)
        elif self.model_selector_card.combo.count() > 0:
            self.model_selector_card.combo.setCurrentIndex(0)
        
        self.model_selector_card.combo.blockSignals(False)

        if self.model_selector_card.combo.currentText():
            self._on_learning_model_selected(self.model_selector_card.combo.currentText())


from PyQt5.QtWidgets import (
    QDialog, QVBoxLayout, QHBoxLayout, QLabel, 
    QFrame, QPushButton, QScrollArea, QWidget,
    QTabWidget, QTextEdit
)
from PyQt5.QtCore import Qt
from PyQt5.QtGui import QFont

class ManualDialog(QDialog):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.setWindowTitle("MASPY GUI - User Manual")
        self.setMinimumSize(900, 650)
        
        self.setWindowFlags(
            Qt.Window |
            Qt.WindowMinimizeButtonHint |
            Qt.WindowMaximizeButtonHint |
            Qt.WindowCloseButtonHint
        )
        
        self._setup_ui()

    def _setup_ui(self):
        main_layout = QVBoxLayout(self)
        main_layout.setContentsMargins(20, 20, 20, 20)
        main_layout.setSpacing(15)

        title = QLabel("MASPY GUI - User Manual")
        title.setProperty("class", "h1")
        title.setFont(QFont("Arial", 18, QFont.Bold))
        main_layout.addWidget(title)

        self.tabs = QTabWidget()
        self.tabs.addTab(self._create_overview_tab(), "Overview")
        self.tabs.addTab(self._create_interface_tab(), "Interface Guide")
        self.tabs.addTab(self._create_maspy_tab(), "MASPY Reference")
        
        self.tabs.setStyleSheet("""
            QTabBar::tab {
                min-width: 180px;
                min-height: 30px;
                padding: 8px 16px;
                font-size: 20px;
            }
        """)
        
        main_layout.addWidget(self.tabs, stretch=1)

        close_btn = QPushButton("Close")
        close_btn.setFixedWidth(120)
        close_btn.clicked.connect(self.accept)
        
        btn_layout = QHBoxLayout()
        btn_layout.addStretch()
        btn_layout.addWidget(close_btn)
        main_layout.addLayout(btn_layout)

    def _create_scrollable_content(self, html_content):
        scroll = QScrollArea()
        scroll.setWidgetResizable(True)
        scroll.setFrameShape(QFrame.NoFrame)
        
        content_widget = QWidget()
        content_layout = QVBoxLayout(content_widget)
        content_layout.setContentsMargins(10, 10, 10, 10)
        
        text_edit = QTextEdit()
        text_edit.setReadOnly(True)
        text_edit.setFrameShape(QFrame.NoFrame)

        styled_html = f"<div style='font-size: 20px;'>{html_content}</div>"
        text_edit.setHtml(styled_html)
        
        
        content_layout.addWidget(text_edit)
        scroll.setWidget(content_widget)
        
        return scroll

    def _create_overview_tab(self):
        html = """
        <h2>MASPY GUI</h2>
        <p>This interface is used for monitoring and debugging the execution of MASPY</p>
        
        <p><b>Main Features:</b></p>
        <ul>
            <li><b>Timeline Navigation:</b> Pause and navigate through the simulation history</li>
            <li><b>Agent Inspection:</b> View detailed information about each agent, monitoring agent behaviours, goals and percepts history</li>
            <li><b>Environment Tracking:</b> Monitor environment changes and percepts history</li>
            <li><b>Message Logging:</b> Analyze communication between agents</li>
            <li><b>Learning Visualization:</b> Observe Q-Learning models</li>
        </ul>
        
        <p><b>Tutorial guide:</b></p>
        <p>Use the <b>sidebar</b> on the left to navigate between different pages
        <p>The <b>timeline slider</b> at the top allows you to rewind and review past simulation states at a specific point in time
        """
        return self._create_scrollable_content(html)

    def _create_interface_tab(self):
        html = """
        <h2>Menu</h2>
        <p>The main page showing an overview of the simulation</p>
        <ul>
            <li><b>Intentions Report:</b> A chronological log of all intentions in the system
            Shows which agent created the intention, what the trigger was, and the intention details.</li>
            <li><b>Pause/Play:</b> When pressed will pause the simulation, press again to continue the execution. Note that if the simulation has ended this button will have no effect</li>
        </ul>
        
        <h2>Agents</h2>
        <p>View all agents in the system displayed as cards</p>
        <ul>
            <li><b>Agent Card:</b> Show agent name, cycle count, beliefs/goals count, and current action</li>
            <li><b>Search:</b> Filter agents by name</li>
            <li><b>View Details:</b> Click to open a detailed window with internal agent information</li>
            <ul>
                <li><b>Overview</b></li>
                <ul>
                    <li><b>Real Time Status:</b> Shows the current cycle, current action, processing event and number of intentions</li>
                    <li><b>Beliefs:</b> Shows the agent's beliefs</li>
                    <li><b>Running Intentions:</b> Shows the agent's intentions</li>
                    <li><b>Goals:</b> Shows the agent's goals</li>
                </ul>
                <li><b>Agent's mind</b></li>
                <ul>
                    <li><b>Beliefs History:</b> Shows all beliefs of the agent during system execution</li>
                    <li><b>Goals History:</b> Shows all goals of the agent during system execution</li>
                    <li><b>Intentions History:</b> Shows all intentions of the agent during system execution</li>
                </ul>
                <li><b>Environment and Channels</b></li>
                <ul>
                    <li><b>Perceptions:</b> Shows the environment perception state that the agent is connected to</li>
                    <li><b>Environments:</b> Shows the list of environments the agent is connected to</li>
                    <li><b>Channels:</b> Shows the list of channels the agent is connected to</li>
                </ul>
                <li><b>Learning</b></li>
                <ul>
                    <li><b>Q-Table:</b> Displays the agent's Q-Learning table with state-action values</li>
                    <li><b>Training Status:</b> Shows whether the learning model is still training or has completed</li>
                </ul>
            </ul>
        </ul>
        
        <h2>Environment</h2>
        <p>Monitor the environments in your multiagent system</p>
        <ul>
            <li><b>Connected Agents:</b> Shows which agents are connected to the environment</li>
            <li><b>Percepts:</b> Current state of the environment's percepts</li>
            <li><b>Changes History:</b> Log of all modifications in the percpetion system, showing the action type (CREATE, DELETE, CHANGE), the content of the change, and the action that triggered it</li>
        </ul>
        
        <h2>Messages</h2>
        <p>View all inter-agent communication</p>
        <ul>
            <li><b>Message Log:</b> All messages exchanged between agents</li>
            <li><b>Filters:</b> Filter by sender, receiver, or performative type</li>
        </ul>
        
        <h2>Learning</h2>
        <p>Visualize reinforcement learning models</p>
        <ul>
            <li><b>Models List:</b> All learning models in the system</li>
            <li><b>Q-Table:</b> The policy table showing state-action values</li>
            <li><b>Training Status:</b> Whether training is complete or in progress</li>
        </ul>
        """
        return self._create_scrollable_content(html)

    def _create_maspy_tab(self):
        html = """
        <h2>Multi-Agent Systems</h2>
        <p>A Multi-Agent System is a computational system composed of multiple autonomous agents that interact with each other and are situated in an environment</p>
        
        <p><b>Key characteristics of MAS:</b></p>
        <ul>
            <li><b>Autonomy:</b> Agents operate independently without direct external control</li>
            <li><b>Social Ability:</b> Agents communicate with each other</li>
            <li><b>Reactivity:</b> Agents perceive and respond to changes in the environmen</li>
        </ul>
        
        <h2>BDI Architecture</h2>
        <p>MASPY implements the BDI (Belief-Desire-Intention) architecture, a philosophical and computational model for rational agents. 
        This architecture models the practical reasoning process of agents based on three key components:</p>
        
        <p><b>Beliefs</b></p>
        <p>Beliefs represent the agent's knowledge about itself, other agents, and the environment. 
        They are the informational component of the agent's state and can be updated through perception or inference.</p>
        <ul>
            <li>Beliefs can be added, removed or modified during execution</li>
            <li>Changes in beliefs can trigger plans and generate new intentions</li>
        </ul>
        
        <p><b>Desires (Goals)</b></p>
        <p>Desires represent the motivational state of the agent, the objectives it wants to achieve.
        In MASPY, desires are implemented as Goals that the agent actively pursues.</p>
        <ul>
            <li>Goals define what the agent wants to accomplish</li>
            <li>Goals trigger the selection of appropriate plans</li>
        </ul>
        
        <p><b>Intentions</b></p>
        <p>Intentions represent the deliberative component, the plans the agent has committed to execute.</p>
        <ul>
            <li>Intentions are instantiated plans being executed</li>
            <li>Multiple intentions can run concurrently</li>
        </ul>
        
        <h2>MASPY Framework</h2>
        <p>MASPY is a Python framework for developing Multi-Agent Systems based on the BDI architecture. 
        It provides a structured way to define agents, environments, and communication channels.</p>
        
        <h3>Agents</h3>
        <p>In MASPY, agents are autonomous entities that follow the BDI reasoning cycle:</p>
        <ul>
            <li><b>Perception:</b> The agent perceives the environment through percepts</li>
            <li><b>Belief Update:</b> Percepts are processed and beliefs are updated accordingly</li>
            <li><b>Deliberation:</b> The agent selects which goals to pursue based on current beliefs</li>
            <li><b>Plan Selection:</b> Appropriate plans are selected to achieve the chosen goals</li>
            <li><b>Execution:</b> The agent executes actions from the selected plans</li>
        </ul>
        
        <p><b>Plans</b></p>
        <p>Plans define how agents achieve their goals. Each plan consists of:</p>
        <ul>
            <li><b>Trigger:</b> The event that activates the plan </li>
            <li><b>Context:</b> Conditions that need to exist for the plan to be applicable</li>
            <li><b>Body:</b> The sequence of actions and the plan itself</li>
        </ul>
        
        <h3>Environment</h3>
        <p>The Environment in MASPY is the shared space where agents perceive and act. 
        It manages percepts that agents can sense and actions that modify the world state.
        It is possible to create multiple environments with multiple agents</p>
        <ul>
            <li><b>Percepts:</b> Information available in the environment that agents can perceive</li>
            <li><b>Actions:</b> Operations that agents can perform to modify the environment</li>
            <li><b>Multiple agents</b> can be connected to the same environment</li>
        </ul>
        
        <h3>Communication</h3>
        <p>MASPY provides inter-agent communication through message passing using messages:</p>
        
        <p><b>Messages</b></p>
        <p>Messages are structured communications between agents containing:</p>
        <ul>
            <li><b>Sender:</b> The agent sending the message</li>
            <li><b>Receiver:</b> The target agent(s) for the message</li>
            <li><b>Performative:</b> The type of communicative: tell (Belief), achieve (Goal), ask</li>
            <li><b>Content:</b> The actual information being communicated</li>
        </ul>
        
        <p><b>Channels</b></p>
        <p>Channels are communication pathways that connect agents:</p>
        <ul>
            <li>Agents must be connected to a channel to send/receive messages</li>
            <li>Channels can be used for group communication</li>
            <li>Different channels can serve different communication purposes</li>
        </ul>
        
        <p><b>Performatives</b></p>
        <p>Performatives define the intent of a message:</p>
        <ul>
            <li><b>achieve:</b> Request that a goal be achieved</li>
            <li><b>tell:</b> Send a belief to another agent</li>
            <li><b>askOne/askAll:</b> Ask for information</li>
        </ul>
        
        <h3>Admin (ADM)</h3>
        <p>The Admin component manages the overall execution of the multi-agent system:</p>
        <ul>
            <li><b>Configuration:</b> System settings and initialization parameters</li>
        </ul>
        
        """
        return self._create_scrollable_content(html)


from gui.assets.theme.theme_dark import theme_colors as theme_dark
from gui.assets.theme.theme_light import theme_colors as theme_light
from pathlib import Path

current_theme = theme_dark.copy()

def set_active_theme(mode="dark"):
    current_theme.clear()
    
    if mode == "light":
        current_theme.update(theme_light)
    else:
        current_theme.update(theme_dark)

def load_stylesheet(theme_name="dark"):
    try:
        current_script_dir = Path(__file__).resolve().parent.parent
        with open(f'{current_script_dir}/styles/theme.qss', 'r', encoding='utf-8') as f:
            stylesheet = f.read()

        target_theme = theme_light if theme_name == "light" else theme_dark
            
        for color_name, color_value in target_theme.items():
            placeholder = f"{{{color_name}}}"
            stylesheet = stylesheet.replace(placeholder, color_value)
        
        return stylesheet
    except Exception as e:
        print(f"Erro style: {e}")
        return ""


from PyQt5.QtWidgets import QFrame, QVBoxLayout, QPushButton, QButtonGroup, QLabel
from PyQt5.QtCore import Qt

class SideBar(QFrame):
    def __init__(self, parent=None):
        super().__init__(parent)

        self.setProperty("class", "sidebar")

        self.setFixedWidth(180)
        layout = QVBoxLayout()
        layout.setContentsMargins(10, 10, 10, 10)
        layout.setSpacing(5)
        layout.setAlignment(Qt.AlignTop)

        self.button_group = QButtonGroup(self)
        self.button_group.setExclusive(True)

        self.buttons = {}
        button_names = ["Menu", "Agents", "Environment", "Messages", "Learning"]

        for text in button_names:
            btn = QPushButton(text)
            btn.setCheckable(True)
            btn.setCursor(Qt.PointingHandCursor)
            btn.setProperty("class", "sidebar-button")
            
            self.buttons[text] = btn
            layout.addWidget(btn)
            self.button_group.addButton(btn)

        version = QLabel("ver.2026.02.06")
        version.setAlignment(Qt.AlignRight)
        version.setProperty("class", "text-secondary")
        layout.addStretch(1)
        layout.addWidget(version)

        self.setLayout(layout)

        


import re
from PyQt5.QtWidgets import (
    QDialog, QVBoxLayout, QHBoxLayout, QLabel, QPushButton,
    QFrame, QListWidget, QListWidgetItem, QGraphicsScene,
    QGraphicsView, QFileDialog, QSplitter, QWidget, QCheckBox,
    QGraphicsLineItem, QGraphicsTextItem, QGraphicsRectItem,
    QAbstractItemView, QMessageBox, QSizePolicy
)
from PyQt5.QtCore import Qt, QRectF, QPointF, QLineF
from PyQt5.QtGui import (
    QFont, QColor, QPen, QBrush, QPainter, QPolygonF, QImage, QPainterPath
)

from gui.assets.theme.styler import current_theme


class InfoCard(QFrame):
    def __init__(self, parent=None):
        super().__init__(parent)
        self._agents_count = 0
        self._messages_count = 0
        
        self.setMinimumHeight(70)
        self.setMaximumHeight(80)
        self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
    
    def set_data(self, agents, messages):
        self._agents_count = agents
        self._messages_count = messages
        self.update()
    
    def paintEvent(self, event):
        painter = QPainter(self)
        painter.setRenderHint(QPainter.Antialiasing)
        
        rect = self.rect()
        
        bg_color = QColor(current_theme.get('background_secondary', '#1F2937'))
        border_color = QColor(current_theme.get('background_tertiary', '#4B5563'))
        text_primary = QColor(current_theme.get('text_primary', '#F9FAFB'))
        text_secondary = QColor(current_theme.get('text_secondary', '#9CA3AF'))
        accent = QColor(current_theme.get('interactive_primary', '#3B82F6'))
        success = QColor(current_theme.get('success', '#10B981'))
        
        bg_rect = QRectF(rect).adjusted(1, 1, -1, -1)
        path = QPainterPath()
        path.addRoundedRect(bg_rect, 8, 8)
        
        painter.setPen(QPen(border_color, 1))
        painter.setBrush(QBrush(bg_color))
        painter.drawPath(path)
        
        col_width = bg_rect.width() / 2
        
        label_font = QFont("Segoe UI", 9)
        value_font = QFont("Segoe UI", 14, QFont.Bold)
        
        x1 = bg_rect.left() + 16
        painter.setFont(label_font)
        painter.setPen(text_secondary)
        painter.drawText(QRectF(x1, bg_rect.top() + 12, col_width, 16), 
                        Qt.AlignLeft | Qt.AlignVCenter, "Selected Agents")
        
        painter.setFont(value_font)
        painter.setPen(accent)
        painter.drawText(QRectF(x1, bg_rect.top() + 32, col_width, 28), 
                        Qt.AlignLeft | Qt.AlignVCenter, str(self._agents_count))
        
        # Coluna 2: Messages
        x2 = bg_rect.left() + col_width
        painter.setFont(label_font)
        painter.setPen(text_secondary)
        painter.drawText(QRectF(x2, bg_rect.top() + 12, col_width, 16), 
                        Qt.AlignLeft | Qt.AlignVCenter, "Messages Found")
        
        painter.setFont(value_font)
        painter.setPen(success)
        painter.drawText(QRectF(x2, bg_rect.top() + 32, col_width, 28), 
                        Qt.AlignLeft | Qt.AlignVCenter, str(self._messages_count))
        
        painter.end()


class MessageDiagramDialog(QDialog):
    def __init__(self, log_store, parent=None):
        super().__init__(parent)
        self.log_store = log_store
        self.selected_agents = []
        self.all_messages = []
        
        self.setWindowTitle("Message Sequence Diagram")
        self.setMinimumSize(1100, 750)
        self.setWindowFlags(
            Qt.Window |
            Qt.WindowMinimizeButtonHint |
            Qt.WindowMaximizeButtonHint |
            Qt.WindowCloseButtonHint
        )
        
        self._apply_dialog_style()
        self._setup_ui()
        self._load_agents()
    
    def _apply_dialog_style(self):
        bg_primary = current_theme.get('background_primary', '#111827')
        bg_secondary = current_theme.get('background_secondary', '#1F2937')
        bg_tertiary = current_theme.get('background_tertiary', '#4B5563')
        text_primary = current_theme.get('text_primary', '#F9FAFB')
        text_secondary = current_theme.get('text_secondary', '#9CA3AF')
        interactive = current_theme.get('interactive_primary', '#3B82F6')
        
        self.setStyleSheet(f"""
            QDialog {{
                background-color: {bg_primary};
                color: {text_primary};
            }}
            QLabel {{
                color: {text_primary};
            }}
            QLabel[class="text-secondary"] {{
                color: {text_secondary};
            }}
            QPushButton {{
                background-color: {bg_secondary};
                border: 1px solid {bg_tertiary};
                border-radius: 6px;
                padding: 8px 16px;
                color: {text_primary};
                font-size: 12px;
                min-height: 32px;
            }}
            QPushButton:hover {{
                background-color: {bg_tertiary};
                border-color: {interactive};
            }}
            QPushButton:pressed {{
                background-color: {interactive};
            }}
            QPushButton:disabled {{
                opacity: 0.5;
                color: {text_secondary};
            }}
            QPushButton#primaryButton {{
                background-color: {interactive};
                border: none;
            }}
            QPushButton#primaryButton:hover {{
                background-color: #2563EB;
            }}
            QListWidget {{
                background-color: {bg_secondary};
                border: 1px solid {bg_tertiary};
                border-radius: 8px;
                padding: 4px;
                outline: none;
            }}
            QListWidget::item {{
                background-color: transparent;
                border-radius: 4px;
                padding: 8px 12px;
                margin: 2px 0;
            }}
            QListWidget::item:selected {{
                background-color: {interactive};
            }}
            QListWidget::item:hover:!selected {{
                background-color: {bg_tertiary};
            }}
            QGraphicsView {{
                background-color: {bg_secondary};
                border: 1px solid {bg_tertiary};
                border-radius: 8px;
            }}
            QSplitter::handle {{
                background-color: {bg_tertiary};
                width: 2px;
            }}
        """)
    
    def _setup_ui(self):
        main_layout = QVBoxLayout(self)
        main_layout.setContentsMargins(20, 20, 20, 20)
        main_layout.setSpacing(16)
        
        header_layout = QHBoxLayout()
        
        title = QLabel("Message Sequence Diagram")
        title.setFont(QFont("Segoe UI", 20, QFont.Bold))
        
        subtitle = QLabel("Generate visual diagrams of agent communication")
        subtitle.setProperty("class", "text-secondary")
        subtitle.setFont(QFont("Segoe UI", 11))
        
        title_layout = QVBoxLayout()
        title_layout.setSpacing(2)
        title_layout.addWidget(title)
        title_layout.addWidget(subtitle)
        
        header_layout.addLayout(title_layout)
        header_layout.addStretch()
        
        main_layout.addLayout(header_layout)
        
        self.info_card = InfoCard()
        main_layout.addWidget(self.info_card)
        
        splitter = QSplitter(Qt.Horizontal)
        
        left_panel = self._create_left_panel()
        splitter.addWidget(left_panel)
        
        right_panel = self._create_right_panel()
        splitter.addWidget(right_panel)
        
        splitter.setStretchFactor(0, 1)
        splitter.setStretchFactor(1, 4)
        
        main_layout.addWidget(splitter, stretch=1)
        
        actions_layout = QHBoxLayout()
        actions_layout.setSpacing(12)
        
        self.btn_export_png = QPushButton("Export PNG")
        self.btn_export_png.setObjectName("primaryButton")
        self.btn_export_png.clicked.connect(lambda: self._export_diagram("png"))
        self.btn_export_png.setEnabled(False)
        self.btn_export_png.setMinimumWidth(120)
        
        self.btn_export_svg = QPushButton("Export SVG")
        self.btn_export_svg.clicked.connect(lambda: self._export_diagram("svg"))
        self.btn_export_svg.setEnabled(False)
        self.btn_export_svg.setMinimumWidth(120)
        
        self.btn_close = QPushButton("Close")
        self.btn_close.clicked.connect(self.accept)
        self.btn_close.setMinimumWidth(100)
        
        actions_layout.addWidget(self.btn_export_png)
        actions_layout.addWidget(self.btn_export_svg)
        actions_layout.addStretch()
        actions_layout.addWidget(self.btn_close)
        
        main_layout.addLayout(actions_layout)
    
    def _create_left_panel(self):
        panel = QFrame()
        panel.setProperty("class", "card")
        panel.setMinimumWidth(220)
        panel.setMaximumWidth(280)
        layout = QVBoxLayout(panel)
        layout.setContentsMargins(16, 16, 16, 16)
        layout.setSpacing(12)
        
        header = QLabel("Select Agents")
        header.setFont(QFont("Segoe UI", 12, QFont.Bold))
        layout.addWidget(header)
        
        hint = QLabel("Select at least 2 agents to generate diagram")
        hint.setProperty("class", "text-secondary")
        hint.setFont(QFont("Segoe UI", 9))
        hint.setWordWrap(True)
        layout.addWidget(hint)
        
        self.agents_list = QListWidget()
        self.agents_list.setSelectionMode(QAbstractItemView.MultiSelection)
        self.agents_list.itemSelectionChanged.connect(self._on_selection_changed)
        layout.addWidget(self.agents_list, 1)
        
        btns_layout = QHBoxLayout()
        btns_layout.setSpacing(8)
        
        self.btn_select_all = QPushButton("Select All")
        self.btn_select_all.clicked.connect(self._select_all_agents)
        
        self.btn_clear = QPushButton("Clear")
        self.btn_clear.clicked.connect(self._clear_selection)
        
        btns_layout.addWidget(self.btn_select_all)
        btns_layout.addWidget(self.btn_clear)
        layout.addLayout(btns_layout)
        
        return panel
    
    def _create_right_panel(self):
        panel = QFrame()
        panel.setProperty("class", "card")
        layout = QVBoxLayout(panel)
        layout.setContentsMargins(16, 16, 16, 16)
        layout.setSpacing(12)
        
        header_layout = QHBoxLayout()
        
        diagram_title = QLabel("Diagram Preview")
        diagram_title.setFont(QFont("Segoe UI", 12, QFont.Bold))
        
        header_layout.addWidget(diagram_title)
        header_layout.addStretch()
        layout.addLayout(header_layout)
        
        self.scene = QGraphicsScene()
        self.view = QGraphicsView(self.scene)
        self.view.setRenderHint(QPainter.Antialiasing)
        self.view.setRenderHint(QPainter.TextAntialiasing)
        self.view.setDragMode(QGraphicsView.ScrollHandDrag)
        layout.addWidget(self.view, 1)
        
        zoom_layout = QHBoxLayout()
        zoom_layout.setSpacing(8)
        
        zoom_label = QLabel("Zoom")
        zoom_label.setProperty("class", "text-secondary")
        zoom_label.setFont(QFont("Segoe UI", 10))
        
        self.btn_zoom_out = QPushButton("-")
        self.btn_zoom_out.setFixedSize(36, 36)
        self.btn_zoom_out.clicked.connect(lambda: self.view.scale(0.8, 0.8))
        
        self.btn_zoom_in = QPushButton("+")
        self.btn_zoom_in.setFixedSize(36, 36)
        self.btn_zoom_in.clicked.connect(lambda: self.view.scale(1.2, 1.2))
        
        self.btn_zoom_reset = QPushButton("Reset")
        self.btn_zoom_reset.clicked.connect(self._reset_zoom)
        
        self.btn_fit = QPushButton("Fit")
        self.btn_fit.clicked.connect(self._fit_to_view)
        
        zoom_layout.addWidget(zoom_label)
        zoom_layout.addWidget(self.btn_zoom_out)
        zoom_layout.addWidget(self.btn_zoom_in)
        zoom_layout.addWidget(self.btn_zoom_reset)
        zoom_layout.addWidget(self.btn_fit)
        zoom_layout.addStretch()
        layout.addLayout(zoom_layout)
        
        return panel
    
    def _load_agents(self):
        messages = self.log_store.get_all_messages()
        agents = set()
        
        for msg in messages:
            agents.add(msg.sender)
            recv_str = msg.receiver
            if "[" in recv_str:
                found = re.findall(r"[\"']?([\w\.-]+)[\"']?", recv_str)
                for x in found:
                    if x not in ['broadcast', '[]', 'N/A']:
                        agents.add(x)
            elif recv_str not in ['broadcast', '[]', 'N/A', 'Unknown']:
                agents.add(recv_str)
        
        for agent in sorted(agents):
            if agent:
                item = QListWidgetItem(agent)
                self.agents_list.addItem(item)
    
    def _on_selection_changed(self):
        selected_items = self.agents_list.selectedItems()
        self.selected_agents = [item.text() for item in selected_items]
        
        has_agents = len(self.selected_agents) >= 2
        self.btn_export_png.setEnabled(has_agents)
        self.btn_export_svg.setEnabled(has_agents)
        
        messages = self._get_filtered_messages()
        self.info_card.set_data(len(self.selected_agents), len(messages))
        
        self._update_diagram()
    
    def _select_all_agents(self):
        self.agents_list.selectAll()
    
    def _clear_selection(self):
        self.agents_list.clearSelection()
    
    def _reset_zoom(self):
        self.view.resetTransform()
    
    def _fit_to_view(self):
        self.view.fitInView(self.scene.sceneRect(), Qt.KeepAspectRatio)
    
    def _get_filtered_messages(self):
        if len(self.selected_agents) < 2:
            return []
        
        messages = self.log_store.get_all_messages()
        filtered = []
        
        for msg in messages:
            sender = msg.sender
            recv_str = msg.receiver
            
            if sender not in self.selected_agents:
                continue
            
            receivers = []
            if "[" in recv_str:
                found = re.findall(r"[\"']?([\w\.-]+)[\"']?", recv_str)
                receivers = [x for x in found if x in self.selected_agents]
            elif recv_str in self.selected_agents:
                receivers = [recv_str]
            
            for receiver in receivers:
                filtered.append({
                    'sender': sender,
                    'receiver': receiver,
                    'performative': msg.performative,
                    'content': msg.content,
                    'time': msg.system_time
                })
        
        return filtered
    
    def _update_diagram(self):
        self.scene.clear()
        
        if len(self.selected_agents) < 2:
            text = self.scene.addText("Select at least 2 agents to generate the diagram")
            text.setDefaultTextColor(QColor(current_theme.get('text_secondary', '#9CA3AF')))
            text.setFont(QFont("Segoe UI", 12))
            return
        
        messages = self._get_filtered_messages()
        
        if not messages:
            text = self.scene.addText("No messages found between selected agents")
            text.setDefaultTextColor(QColor(current_theme.get('text_secondary', '#9CA3AF')))
            text.setFont(QFont("Segoe UI", 12))
            return
        
        self._draw_sequence_diagram(messages)
    
    def _draw_sequence_diagram(self, messages):
        AGENT_WIDTH = 130
        AGENT_HEIGHT = 44
        AGENT_SPACING = 190
        MESSAGE_SPACING = 55
        TOP_MARGIN = 25
        LEFT_MARGIN = 35
        
        c_bg = QColor(current_theme.get('background_tertiary', '#374151'))
        c_border = QColor(current_theme.get('interactive_primary', '#3B82F6'))
        c_text = QColor(current_theme.get('text_primary', '#F9FAFB'))
        c_text_sec = QColor(current_theme.get('text_secondary', '#9CA3AF'))
        c_line = QColor(current_theme.get('text_disabled', '#6B7280'))
        c_arrow = QColor(current_theme.get('info', '#0EA5E9'))
        
        agents = self.selected_agents
        num_agents = len(agents)
        
        agent_positions = {}
        for i, agent in enumerate(agents):
            x = LEFT_MARGIN + i * AGENT_SPACING
            agent_positions[agent] = x + AGENT_WIDTH / 2
        
        total_height = TOP_MARGIN + AGENT_HEIGHT + 40 + len(messages) * MESSAGE_SPACING + 60
        
        for agent in agents:
            x = agent_positions[agent] - AGENT_WIDTH / 2
            y = TOP_MARGIN
            
            rect_path = QPainterPath()
            rect_path.addRoundedRect(x, y, AGENT_WIDTH, AGENT_HEIGHT, 8, 8)
            
            rect_item = self.scene.addPath(rect_path, QPen(c_border, 2), QBrush(c_bg))
            
            text = self.scene.addText(agent)
            text.setDefaultTextColor(c_text)
            text.setFont(QFont("Segoe UI", 10, QFont.Bold))
            text_width = text.boundingRect().width()
            text.setPos(x + (AGENT_WIDTH - text_width) / 2, y + 10)
        
        line_start_y = TOP_MARGIN + AGENT_HEIGHT
        line_end_y = total_height
        
        pen_lifeline = QPen(c_line, 1.5, Qt.DashLine)
        for agent in agents:
            x = agent_positions[agent]
            line = self.scene.addLine(x, line_start_y, x, line_end_y, pen_lifeline)
        
        current_y = TOP_MARGIN + AGENT_HEIGHT + 45
        
        for msg in messages:
            sender = msg['sender']
            receiver = msg['receiver']
            performative = msg['performative']
            content = msg['content']
            
            if sender not in agent_positions or receiver not in agent_positions:
                continue
            
            x1 = agent_positions[sender]
            x2 = agent_positions[receiver]
            
            going_right = x2 > x1
            
            pen_arrow = QPen(c_arrow, 2)
            line = self.scene.addLine(x1, current_y, x2, current_y, pen_arrow)
            
            arrow_size = 10
            if going_right:
                arrow_x = x2 - arrow_size
                p1 = QPointF(arrow_x, current_y - 5)
                p2 = QPointF(x2, current_y)
                p3 = QPointF(arrow_x, current_y + 5)
            else:
                arrow_x = x2 + arrow_size
                p1 = QPointF(arrow_x, current_y - 5)
                p2 = QPointF(x2, current_y)
                p3 = QPointF(arrow_x, current_y + 5)
            
            arrow_head = self.scene.addPolygon(
                QPolygonF([p1, p2, p3]),
                QPen(c_arrow),
                QBrush(c_arrow)
            )
            
            content_clean = re.sub(r'\s*\[[^\]]*\]\s*$', '', content)
            content_short = content_clean[:35] + "..." if len(content_clean) > 35 else content_clean
            msg_text = f"[{performative}] {content_short}"
            
            text_item = self.scene.addText(msg_text)
            text_item.setDefaultTextColor(c_text_sec)
            text_item.setFont(QFont("Segoe UI", 9))
            
            text_width = text_item.boundingRect().width()
            text_x = min(x1, x2) + abs(x2 - x1) / 2 - text_width / 2
            text_item.setPos(text_x, current_y - 22)
            
            current_y += MESSAGE_SPACING
        
        total_width = LEFT_MARGIN + num_agents * AGENT_SPACING + 50
        self.scene.setSceneRect(0, 0, total_width, total_height)
    
    def _export_diagram(self, format_type):
        if format_type == "png":
            file_filter = "PNG Image (*.png)"
            default_ext = ".png"
        else:
            file_filter = "SVG Image (*.svg)"
            default_ext = ".svg"
        
        file_path, _ = QFileDialog.getSaveFileName(
            self,
            "Export Diagram",
            f"message_diagram{default_ext}",
            file_filter
        )
        
        if not file_path:
            return
        
        if format_type == "png":
            self._export_as_png(file_path)
        else:
            self._export_as_svg(file_path)
    
    def _export_as_png(self, file_path):
        scene_rect = self.scene.sceneRect()
        
        scale = 2
        image = QImage(
            int(scene_rect.width() * scale),
            int(scene_rect.height() * scale),
            QImage.Format_ARGB32
        )
        
        bg_color = QColor(current_theme.get('background_primary', '#111827'))
        image.fill(bg_color)
        
        painter = QPainter(image)
        painter.setRenderHint(QPainter.Antialiasing)
        painter.setRenderHint(QPainter.TextAntialiasing)
        painter.scale(scale, scale)
        self.scene.render(painter)
        painter.end()
        
        if image.save(file_path):
            QMessageBox.information(self, "Export Successful", f"Diagram exported to:\n{file_path}")
        else:
            QMessageBox.warning(self, "Export Failed", "Failed to export the diagram.")
    
    def _export_as_svg(self, file_path):
        from PyQt5.QtSvg import QSvgGenerator
        
        scene_rect = self.scene.sceneRect()
        
        generator = QSvgGenerator()
        generator.setFileName(file_path)
        generator.setSize(scene_rect.size().toSize())
        generator.setViewBox(scene_rect)
        generator.setTitle("Message Sequence Diagram")
        
        painter = QPainter(generator)
        painter.setRenderHint(QPainter.Antialiasing)
        painter.setRenderHint(QPainter.TextAntialiasing)
        self.scene.render(painter)
        painter.end()
        
        QMessageBox.information(self, "Export Successful", f"Diagram exported to:\n{file_path}")


from PyQt5.QtCore import QAbstractListModel, Qt, QModelIndex, QSize
from PyQt5.QtGui import QColor, QPainter, QFontMetrics
from PyQt5.QtWidgets import QStyledItemDelegate, QStyle, QListView

from gui.assets.theme.styler import current_theme

class HistoryItem:
    __slots__ = ('type', 'content', 'time', 'cycle')

    def __init__(self, msg_type, content, time, cycle):
        self.type = msg_type  
        self.content = content
        self.time = time
        self.cycle = cycle


class HistoryListView(QListView):
    
    def __init__(self, parent=None):
        super().__init__(parent)
        self._max_width = 0
        self._needs_width_update = True
    
    def setModel(self, model):
        super().setModel(model)
        if model:
            model.rowsInserted.connect(self._on_rows_changed)
            model.modelReset.connect(self._on_model_reset)
        self._needs_width_update = True
    
    def _on_rows_changed(self, parent, first, last):
        model = self.model()
        delegate = self.itemDelegate()
        if not model or not delegate:
            return
        
        option = self.viewOptions()
        for row in range(first, last + 1):
            index = model.index(row, 0)
            size = delegate.sizeHint(option, index)
            self._max_width = max(self._max_width, size.width())
        
        self._update_horizontal_scroll_range()
    
    def _on_model_reset(self):
        self._max_width = 0
        self._needs_width_update = True
    
    def updateGeometries(self):
        super().updateGeometries()
        if self._needs_width_update:
            self._recalculate_max_width()
            self._needs_width_update = False
        self._update_horizontal_scroll_range()
    
    def _recalculate_max_width(self):
        model = self.model()
        delegate = self.itemDelegate()
        
        if not model or not delegate:
            return
        
        max_width = 0
        option = self.viewOptions()
        
        for row in range(model.rowCount()):
            index = model.index(row, 0)
            size = delegate.sizeHint(option, index)
            max_width = max(max_width, size.width())
        
        self._max_width = max_width
    
    def _update_horizontal_scroll_range(self):
        hsb = self.horizontalScrollBar()
        if hsb:
            viewport_width = self.viewport().width()
            if self._max_width > viewport_width:
                hsb.setRange(0, self._max_width - viewport_width)
            else:
                hsb.setRange(0, 0)

class HistoryLogModel(QAbstractListModel):
    def __init__(self, parent=None):
        super().__init__(parent)
        self._data = []

    def rowCount(self, parent=QModelIndex()):
        return len(self._data)

    def data(self, index, role=Qt.DisplayRole):
        if not index.isValid() or not (0 <= index.row() < len(self._data)):
            return None
        
        if role == Qt.UserRole:
            return self._data[index.row()]
        
        return None

    def set_items(self, new_items):
        self.beginResetModel()
        self._data = new_items
        self.endResetModel()

    def clear(self):
        self.beginResetModel()
        self._data.clear()
        self.endResetModel()

class HistoryLogDelegate(QStyledItemDelegate):
    def __init__(self, parent=None, theme_colors_dict=None):
        super().__init__(parent)
        self.padding_left = 5
        self.padding_top = 6
        
        self._update_colors()

    def update_theme_colors(self, new_theme_dict=None):
        self._update_colors()

    def _update_colors(self):
        self.c_gain = QColor(current_theme.get("success", "#10B981"))
        self.c_lose = QColor(current_theme.get("danger", "#EF4444"))
        self.c_info = QColor(current_theme.get("info", "#0EA5E9"))

        self.c_time = QColor(current_theme.get("text_secondary", "#9CA3AF"))
        self.c_text = QColor(current_theme.get("text_primary", "#F9FAFB"))
        self.c_bg = QColor(current_theme.get("background_primary", "#111827"))
        self.c_bg_sel = QColor(current_theme.get("background_secondary", "#1F2937"))

    def paint(self, painter, option, index):
        item = index.data(Qt.UserRole)
        if not item:
            return

        painter.save()
        rect = option.rect
        
        bg_color = self.c_bg
        if option.state & QStyle.State_Selected:
            bg_color = self.c_bg_sel
        elif option.state & QStyle.State_MouseOver:
            bg_color = self.c_bg.lighter(110)
        painter.fillRect(rect, bg_color)
        
        font_main = option.font
        font_bold = option.font
        font_bold.setBold(True)
        
        fm = QFontMetrics(font_main)
        fm_bold = QFontMetrics(font_bold)

        text_y = rect.top() + self.padding_top + fm.ascent()
        current_x = rect.left() + self.padding_left

        time_str = f"[{item.time}][C:{item.cycle}] "
        painter.setFont(font_main)
        painter.setPen(self.c_time)
        painter.drawText(current_x, text_y, time_str)
        current_x += fm.width(time_str)

        type_label = item.type
        color = self.c_text
        
        if item.type == 'GAIN': 
            color = self.c_gain
            type_label = "[GAIN]"
        elif item.type == 'LOSE': 
            color = self.c_lose
            type_label = "[LOSE]"
        elif item.type == 'SUCCESS': 
            color = self.c_gain
            type_label = "[SUCCESS]"
        elif item.type == 'UPDATE': 
            color = self.c_info
            type_label = "[UPDATE]"
        elif item.type == 'INTENTION':
            color = self.c_info
            type_label = ""
            
        type_str = f"{type_label} "
        painter.setFont(font_bold)
        painter.setPen(color)
        painter.drawText(current_x, text_y, type_str)
        current_x += fm_bold.width(type_str)

        content_str = item.content
        painter.setFont(font_main)
        painter.setPen(self.c_text)
        painter.drawText(current_x, text_y, content_str)

        painter.restore()

    def sizeHint(self, option, index):
        item = index.data(Qt.UserRole)
        if not item:
            return QSize(100, option.fontMetrics.height() + (self.padding_top * 2))
        
        fm = option.fontMetrics
        
        font_bold = option.font
        font_bold.setBold(True)
        fm_bold = QFontMetrics(font_bold)
        
        time_str = f"[{item.time}][C:{item.cycle}] "
        type_label = ""
        if item.type == 'GAIN': 
            type_label = "[GAIN] "
        elif item.type == 'LOSE': 
            type_label = "[LOSE] "
        elif item.type == 'SUCCESS': 
            type_label = "[SUCCESS] "
        elif item.type == 'UPDATE': 
            type_label = "[UPDATE] "
        
        total_width = self.padding_left + fm.width(time_str) + fm_bold.width(type_label) + fm.width(item.content) + 100
        height = fm.height() + (self.padding_top * 2)
        
        return QSize(total_width, height)


from PyQt5.QtWidgets import QGraphicsDropShadowEffect
from PyQt5.QtGui import QColor

def apply_shadow(widget, blur_radius=20, offset_x=0, offset_y=5, color="#000000"):
    shadow = QGraphicsDropShadowEffect(widget)
    shadow.setBlurRadius(blur_radius)
    shadow.setColor(QColor(color))
    shadow.setOffset(offset_x, offset_y)
    widget.setGraphicsEffect(shadow)


import math
from PyQt5.QtWidgets import (
    QWidget, QLabel, QVBoxLayout, QHBoxLayout, QPushButton, 
    QFrame, QListView, QAbstractItemView, QStyledItemDelegate, QSizePolicy,
    QGridLayout
) 
from PyQt5.QtGui import QIcon, QFont, QColor, QPainter, QPen, QFontMetrics, QPainterPath, QBrush
from PyQt5.QtCore import Qt, QSize, QAbstractListModel, QModelIndex, QRectF
from gui.assets.theme.styler import current_theme
from gui.widgets.help_button import HelpButton
from pathlib import Path


class MetricCard(QFrame):
    def __init__(self, title, accent_color=None, parent=None):
        super().__init__(parent)
        self.title = title
        self.accent_color = accent_color or "#3B82F6"
        self._value = "0"
        
        self.setMinimumSize(140, 80)
        self.setMaximumHeight(90)
        self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
        
    def set_value(self, value):
        self._value = str(value)
        self.update()
    
    def set_accent_color(self, color):
        self.accent_color = color
        self.update()
        
    def paintEvent(self, event):
        painter = QPainter(self)
        painter.setRenderHint(QPainter.Antialiasing)
        
        rect = self.rect()
        
        bg_color = QColor(current_theme.get('background_secondary', '#1F2937'))
        border_color = QColor(current_theme.get('background_tertiary', '#4B5563'))
        text_primary = QColor(current_theme.get('text_primary', '#F9FAFB'))
        text_secondary = QColor(current_theme.get('text_secondary', '#9CA3AF'))
        accent = QColor(self.accent_color)
        
        bg_rect = QRectF(rect).adjusted(1, 1, -1, -1)
        path = QPainterPath()
        path.addRoundedRect(bg_rect, 8, 8)
        
        painter.setPen(QPen(border_color, 1))
        painter.setBrush(QBrush(bg_color))
        painter.drawPath(path)
        
        accent_rect = QRectF(bg_rect.left(), bg_rect.top() + 8, 4, bg_rect.height() - 16)
        accent_path = QPainterPath()
        accent_path.addRoundedRect(accent_rect, 2, 2)
        painter.setPen(Qt.NoPen)
        painter.setBrush(QBrush(accent))
        painter.drawPath(accent_path)
        
        value_font = QFont("Segoe UI", 22, QFont.Bold)
        painter.setFont(value_font)
        painter.setPen(text_primary)
        
        value_rect = QRectF(bg_rect.left() + 16, bg_rect.top() + 12, bg_rect.width() - 24, 32)
        painter.drawText(value_rect, Qt.AlignLeft | Qt.AlignVCenter, self._value)
        
        title_font = QFont("Segoe UI", 10)
        painter.setFont(title_font)
        painter.setPen(text_secondary)
        
        title_rect = QRectF(bg_rect.left() + 16, bg_rect.bottom() - 28, bg_rect.width() - 24, 20)
        painter.drawText(title_rect, Qt.AlignLeft | Qt.AlignVCenter, self.title)
        
        painter.end()


class PaginatedIntentionModel(QAbstractListModel):
    def __init__(self, log_store, items_per_page=100, parent=None):
        super().__init__(parent)
        self.log_store = log_store
        self.items_per_page = items_per_page
        self.current_page = 0
        self._total_count = 0

    def rowCount(self, parent=QModelIndex()):
        total = len(self.log_store.get_all_intentions_history())
        self._total_count = total
        remaining = total - (self.current_page * self.items_per_page)
        return max(0, min(remaining, self.items_per_page))

    def data(self, index, role=Qt.DisplayRole):
        if not index.isValid(): return None
        if role == Qt.DisplayRole:
            source_list = self.log_store.get_all_intentions_history()
            total = len(source_list)
            
            real_offset = (self.current_page * self.items_per_page) + index.row()
            if real_offset >= total: return None
            
            actual_index = total - 1 - real_offset
            return source_list[actual_index]
        return None

    def set_page(self, page):
        self.beginResetModel()
        self.current_page = page
        self.endResetModel()

    def refresh_if_needed(self):
        if self.current_page == 0:
            self.layoutChanged.emit()

    def get_total_pages(self):
        total = len(self.log_store.get_all_intentions_history())
        return max(1, math.ceil(total / self.items_per_page))


class IntentionDelegateCard(QStyledItemDelegate):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.padding = 18
        self.spacing = 6
        self._update_colors()

    def _update_colors(self):
        self.c_bg_secondary = QColor(current_theme.get('background_secondary', '#1F2937'))
        self.c_bg_tertiary = QColor(current_theme.get('background_tertiary', '#4B5563'))
        self.c_text_pri = QColor(current_theme.get('text_primary', '#F9FAFB'))
        self.c_text_sec = QColor(current_theme.get('text_secondary', '#9CA3AF'))
        self.c_info = QColor(current_theme.get('info', '#0EA5E9')) 

        is_light = current_theme.get('background_primary', '#111827') != "#111827"
        self.c_shadow = QColor("#000000")
        self.c_shadow.setAlpha(60 if is_light else 30)

    def update_theme(self):
        self._update_colors()

    def paint(self, painter, option, index):
        item = index.data(Qt.DisplayRole)
        if not item: return

        try:
            agent_name = getattr(item, 'agent_name', 'N/A')
            time_str = getattr(item, 'system_time', '00:00:00')
            trigger_raw = getattr(item, 'trigger', 'N/A')
            intention = getattr(item, 'intention', 'N/A')
        except AttributeError:
            agent_name = item.get('agent_name', 'N/A')
            time_str = item.get('system_time', '00:00:00.000')
            trigger_raw = item.get('last_event', 'N/A')
            intention = item.get('intention_str', 'N/A')

        trigger = str(trigger_raw).replace('gain:', '').replace('lose:', '')

        painter.save()
        painter.setRenderHint(QPainter.Antialiasing)

        rect = QRectF(option.rect).adjusted(5, 5, -5, -5)

        shadow_rect = rect.translated(0, 1)
        path_shadow = QPainterPath()
        path_shadow.addRoundedRect(shadow_rect, 4, 4)
        painter.setBrush(self.c_shadow)
        painter.setPen(Qt.NoPen)
        painter.drawPath(path_shadow)

        path_bg = QPainterPath()
        path_bg.addRoundedRect(rect, 4, 4)
        painter.setBrush(self.c_bg_secondary)
        painter.setPen(QPen(self.c_bg_tertiary, 1)) 
        painter.drawPath(path_bg)

        f_default = QFont("Segoe UI", 12)
        f_bold = QFont("Segoe UI", 12, QFont.Bold)
        f_italic = QFont("Segoe UI", 11); f_italic.setItalic(True)
        
        fm_bold = QFontMetrics(f_bold)
        fm_italic = QFontMetrics(f_italic)

        content_rect = rect.adjusted(self.padding, self.padding, -self.padding, -self.padding)
        y = content_rect.top()
        w = content_rect.width()

        painter.setFont(f_bold)
        painter.setPen(self.c_info)
        painter.drawText(QRectF(content_rect.left(), y, w, fm_bold.height()), Qt.AlignLeft, agent_name)

        time_txt = f"[{time_str}]"
        painter.setFont(f_default)
        painter.setPen(self.c_text_sec)
        painter.drawText(QRectF(content_rect.left(), y, w, fm_bold.height()), Qt.AlignRight, time_txt)

        y += fm_bold.height() + self.spacing

        painter.setFont(f_italic)
        painter.setPen(self.c_text_sec)
        trig_txt = f"Trigger: {trigger}"
        painter.drawText(QRectF(content_rect.left(), y, w, fm_italic.height()), Qt.AlignLeft, trig_txt)

        y += fm_italic.height() + self.spacing

        painter.setFont(f_bold) 
        painter.setPen(self.c_text_pri)
        
        text_rect = QRectF(content_rect.left(), y, w, content_rect.bottom() - y)
        painter.drawText(text_rect, Qt.AlignLeft | Qt.TextWordWrap, intention)

        painter.restore()

    def sizeHint(self, option, index):
        item = index.data(Qt.DisplayRole)
        if not item: return QSize(0, 0)
        
        try:
            intention = getattr(item, 'intention', '')
        except AttributeError:
            intention = item.get('intention_str', '')

        w = option.rect.width() - 10 - (2 * self.padding)
        if w <= 0: w = 200

        fm = QFontMetrics(QFont("Segoe UI", 12, QFont.Bold))
        h_header = fm.height()
        h_trigger = fm.height()
        
        rect_text = fm.boundingRect(0, 0, int(w), 5000, Qt.AlignLeft | Qt.TextWordWrap, intention)
        
        h_total = (
            10 +
            (2 * self.padding) + 
            h_header + 
            self.spacing + 
            h_trigger + 
            self.spacing + 
            rect_text.height()
        )

        return QSize(option.rect.width(), int(h_total))


class MenuInicialPage(QWidget):
    def __init__(self, command_queue, log_store):
        super().__init__()
        self.setProperty("class", "page")
        self.command_queue = command_queue
        self.log_store = log_store
        self.simulation_paused = False
        self.gui_path = Path(__file__).resolve().parent.parent
        self._setup_ui()

    def _setup_ui(self):
        main_layout = QVBoxLayout(self)
        main_layout.setAlignment(Qt.AlignTop | Qt.AlignLeft)
        main_layout.setContentsMargins(30, 20, 30, 20)
        main_layout.setSpacing(15)

        header_layout = QHBoxLayout()
        
        title_label = QLabel("Simulation Overview")
        title_label.setProperty("class", "h1")
        
        self.pause_button = QPushButton(" Pause")
        self.pause_button.setMinimumHeight(36)
        self.pause_button.setMinimumWidth(120)
        try:
            self.pause_button.setIcon(QIcon(f"{self.gui_path}/assets/icons/pause-circle.svg"))
        except: pass
        self.pause_button.setIconSize(QSize(18, 18))
        self.pause_button.setCursor(Qt.PointingHandCursor)
        self.pause_button.setEnabled(True)
        self.pause_button.clicked.connect(self._toggle_pause_simulation)
        
        header_layout.addWidget(title_label)
        header_layout.addStretch()
        header_layout.addWidget(self.pause_button)
        main_layout.addLayout(header_layout)

        cards_layout = QHBoxLayout()
        cards_layout.setSpacing(12)

        self.card_agents = MetricCard("Agents", "#FF0000")
        self.card_messages = MetricCard("Messages", "#0D00FF") #MUDAR O AZUL, PRO AZUL SER AZUL DO OUTRO AZUL
        self.card_intentions = MetricCard("Intentions", "#F59E0B")
        self.card_environments = MetricCard("Environments", "#15FF00")
        
        cards_layout.addWidget(self.card_agents)
        cards_layout.addWidget(self.card_messages)
        cards_layout.addWidget(self.card_intentions)
        cards_layout.addWidget(self.card_environments)
        
        main_layout.addLayout(cards_layout)

        main_frame = QFrame()
        main_frame.setProperty("class", "card") 
        main_frame_layout = QVBoxLayout(main_frame)
        main_frame_layout.setContentsMargins(20, 20, 20, 20)

        header_layout = QHBoxLayout()
        header_layout.setSpacing(8)
        
        left_title = QLabel("Intentions Report")
        left_title.setProperty("class", "h2")
        left_title.setFont(QFont("Segoe UI", 14, QFont.Bold))
        
        self.help_button = HelpButton(
            "<div style='max-width: 450px;'>"
            "<h3 style='margin: 0 0 10px 0; color: #3B82F6;'>Intentions Report</h3>"
            "<p style='margin: 0 0 10px 0;'>This panel shows a chronological log of all system intentions</p>"
            "<p style='margin: 0 0 5px 0;'><b>Agent Name:</b> The agent that created the intention</p>"
            "<p style='margin: 0 0 5px 0;'><b>Timestamp:</b> When the intention was created</p>"
            "<p style='margin: 0 0 5px 0;'><b>Trigger:</b> The event that triggered the intention, examples include a Goal, Belief or a Percept</p>"
            "</div>",
            icon_size=23
        )

        self.intention_count_label = QLabel("Total: 0")
        self.intention_count_label.setProperty("class", "text-secondary")
        self.intention_count_label.setFont(QFont("Segoe UI", 11))
        
        header_layout.addWidget(left_title)
        header_layout.addWidget(self.help_button)
        header_layout.addStretch()
        header_layout.addWidget(self.intention_count_label)
        main_frame_layout.addLayout(header_layout)

        self.intention_list = QListView()
        self.intention_list.setFrameShape(QFrame.NoFrame)
        self.intention_list.setVerticalScrollMode(QAbstractItemView.ScrollPerPixel)
        self.intention_list.setUniformItemSizes(False)
        self.intention_list.setResizeMode(QListView.Adjust)
        self.intention_list.setStyleSheet("background: transparent; border: none;")
        
        self.model = PaginatedIntentionModel(self.log_store, items_per_page=200)
        self.delegate = IntentionDelegateCard()
        
        self.intention_list.setModel(self.model)
        self.intention_list.setItemDelegate(self.delegate)

        pagination_layout = QHBoxLayout()
        self.btn_prev = QPushButton("« Previous")
        self.btn_prev.clicked.connect(self._prev_page)
        self.btn_prev.setEnabled(False)
        
        self.page_label = QLabel("Page 1 of 1")
        self.page_label.setAlignment(Qt.AlignCenter)
        self.page_label.setFont(QFont("Segoe UI", 11))
        
        self.btn_next = QPushButton("Next »")
        self.btn_next.clicked.connect(self._next_page)
        self.btn_next.setEnabled(False)
        
        pagination_layout.addWidget(self.btn_prev)
        pagination_layout.addStretch()
        pagination_layout.addWidget(self.page_label)
        pagination_layout.addStretch()
        pagination_layout.addWidget(self.btn_next)
        
        main_frame_layout.addWidget(self.intention_list, stretch=1)
        main_frame_layout.addLayout(pagination_layout)
        
        main_layout.addWidget(main_frame, stretch=1)

    def _toggle_pause_simulation(self):
        self.command_queue.put('TOGGLE_PAUSE')
        self.log_store.toggle_live_mode()
        self.simulation_paused = not self.simulation_paused
        
        if self.simulation_paused:
            self.pause_button.setText(" Play")
            try: self.pause_button.setIcon(QIcon(f"{self.gui_path}/assets/icons/play-circle.svg"))
            except: pass
        else:
            self.pause_button.setText(" Pause")
            try: self.pause_button.setIcon(QIcon(f"{self.gui_path}/assets/icons/pause-circle.svg"))
            except: pass
            
    def on_store_updated(self):
        self._update_metrics()

        total = len(self.log_store.get_all_intentions_history())
        if self.model._total_count != total:
             self._update_pagination_ui()
             self.model.refresh_if_needed()

    def _update_metrics(self):
        agents_count = len(self.log_store.known_agents)-1 #por favor, todos que estiverem vendo isso, ignore... desculpa
        self.card_agents.set_value(agents_count)
        
        messages = self.log_store.get_all_messages()
        self.card_messages.set_value(len(messages))
        
        intentions = self.log_store.get_all_intentions_history()
        self.card_intentions.set_value(len(intentions))
        
        env_states = self.log_store.get_environment_states_at_index(
            self.log_store.current_timeline_index
        )
        self.card_environments.set_value(len(env_states) if env_states else 0)

    def _update_pagination_ui(self):
        total = len(self.log_store.get_all_intentions_history())
        total_pages = self.model.get_total_pages()
        current = self.model.current_page
        
        self.page_label.setText(f"Page {current + 1} of {total_pages}")
        self.intention_count_label.setText(f"Total: {total}")
        
        self.btn_prev.setEnabled(current > 0)
        self.btn_next.setEnabled(current < total_pages - 1)

    def _next_page(self):
        if self.model.current_page < self.model.get_total_pages() - 1:
            self.model.set_page(self.model.current_page + 1)
            self._update_pagination_ui()
            self.intention_list.scrollToTop()

    def _prev_page(self):
        if self.model.current_page > 0:
            self.model.set_page(self.model.current_page - 1)
            self._update_pagination_ui()
            self.intention_list.scrollToTop()

    def update_theme(self):
        self.delegate.update_theme()
        self.card_agents.update()
        self.card_messages.update()
        self.card_intentions.update()
        self.card_environments.update()
        
        self.intention_list.viewport().update()


from PyQt5.QtWidgets import (
    QWidget, QHBoxLayout, QVBoxLayout, QSlider, QLabel, 
    QPushButton, QComboBox, QFrame, QSizePolicy, QToolTip
)
from PyQt5.QtCore import Qt, QTimer, pyqtSignal, QPoint
from PyQt5.QtGui import QFont, QPainter, QPen, QBrush, QColor, QPainterPath, QPolygon

from gui.assets.theme.styler import current_theme
from gui.widgets.help_button import HelpButton


class IconButton(QPushButton):
    
    ICON_START = "start"      
    ICON_BACK = "back"        
    ICON_PLAY = "play"        
    ICON_PAUSE = "pause"      
    ICON_FORWARD = "forward"  
    ICON_END = "end"          
    
    def __init__(self, icon_type, parent=None):
        super().__init__(parent)
        self._icon_type = icon_type
        self._hovered = False
        self._pressed = False
        
        self.setFixedSize(36, 36)
        self.setCursor(Qt.PointingHandCursor)
        self.setMouseTracking(True)
    
    def set_icon_type(self, icon_type):
        self._icon_type = icon_type
        self.update()
    
    def enterEvent(self, event):
        self._hovered = True
        self.update()
        super().enterEvent(event)
    
    def leaveEvent(self, event):
        self._hovered = False
        self.update()
        super().leaveEvent(event)
    
    def mousePressEvent(self, event):
        self._pressed = True
        self.update()
        super().mousePressEvent(event)
    
    def mouseReleaseEvent(self, event):
        self._pressed = False
        self.update()
        super().mouseReleaseEvent(event)
    
    def paintEvent(self, event):
        painter = QPainter(self)
        painter.setRenderHint(QPainter.Antialiasing)
        
        rect = self.rect()
        
        bg_color = QColor(current_theme.get('background_secondary', '#1F2937'))
        bg_hover = QColor(current_theme.get('background_tertiary', '#4B5563'))
        bg_pressed = QColor(current_theme.get('interactive_primary', '#3B82F6'))
        border_color = QColor(current_theme.get('background_tertiary', '#4B5563'))
        icon_color = QColor(current_theme.get('text_primary', '#F9FAFB'))
        
        if self._pressed:
            bg = bg_pressed
        elif self._hovered:
            bg = bg_hover
        else:
            bg = bg_color
        
        path = QPainterPath()
        path.addRoundedRect(1, 1, rect.width() - 2, rect.height() - 2, 6, 6)
        
        painter.setPen(QPen(border_color, 1))
        painter.setBrush(QBrush(bg))
        painter.drawPath(path)
        
        painter.setPen(Qt.NoPen)
        painter.setBrush(QBrush(icon_color))
        
        cx = rect.width() / 2
        cy = rect.height() / 2
        
        if self._icon_type == self.ICON_START:
            painter.fillRect(int(cx - 8), int(cy - 6), 3, 12, icon_color)
            self._draw_triangle(painter, cx + 6, cy, 8, "left", icon_color)
            
        elif self._icon_type == self.ICON_BACK:
            self._draw_triangle(painter, cx + 2, cy, 10, "left", icon_color)
            
        elif self._icon_type == self.ICON_PLAY:
            self._draw_triangle(painter, cx - 2, cy, 12, "right", icon_color)
            
        elif self._icon_type == self.ICON_PAUSE:
            painter.fillRect(int(cx - 6), int(cy - 6), 4, 12, icon_color)
            painter.fillRect(int(cx + 2), int(cy - 6), 4, 12, icon_color)
            
        elif self._icon_type == self.ICON_FORWARD:
            self._draw_triangle(painter, cx - 2, cy, 10, "right", icon_color)
            
        elif self._icon_type == self.ICON_END:
            self._draw_triangle(painter, cx - 6, cy, 8, "right", icon_color)
            painter.fillRect(int(cx + 5), int(cy - 6), 3, 12, icon_color)
        
        painter.end()
    
    def _draw_triangle(self, painter, x, y, size, direction, color):
        half = size / 2
        
        if direction == "right":
            points = [
                QPoint(int(x), int(y - half)),
                QPoint(int(x + size), int(y)),
                QPoint(int(x), int(y + half))
            ]
        else:
            points = [
                QPoint(int(x), int(y - half)),
                QPoint(int(x - size), int(y)),
                QPoint(int(x), int(y + half))
            ]
        
        painter.setPen(Qt.NoPen)
        painter.setBrush(QBrush(color))
        painter.drawPolygon(QPolygon(points))


class StyledSlider(QSlider):
    def __init__(self, orientation, parent=None):
        super().__init__(orientation, parent)
        self._format_func = None
        self.setMouseTracking(True)
    
    def set_format_function(self, func):
        self._format_func = func
    
    def mouseMoveEvent(self, event):
        if self.maximum() > 0:
            pos = event.pos().x()
            width = self.width()
            value = int((pos / width) * self.maximum())
            value = max(0, min(value, self.maximum()))
            
            if self._format_func:
                text = self._format_func(value)
            else:
                text = str(value)
            
            QToolTip.showText(event.globalPos(), text, self)
        
        super().mouseMoveEvent(event)
    
    def apply_style(self):
        bg_secondary = current_theme.get('background_secondary', '#1F2937')
        bg_tertiary = current_theme.get('background_tertiary', '#4B5563')
        interactive = current_theme.get('interactive_primary', '#3B82F6')
        
        self.setStyleSheet(f"""
            QSlider::groove:horizontal {{
                background: {bg_tertiary};
                height: 6px;
                border-radius: 3px;
            }}
            QSlider::handle:horizontal {{
                background: {interactive};
                width: 16px;
                height: 16px;
                margin: -5px 0;
                border-radius: 8px;
            }}
            QSlider::handle:horizontal:hover {{
                background: #2563EB;
            }}
            QSlider::sub-page:horizontal {{
                background: {interactive};
                border-radius: 3px;
            }}
        """)


class TimelineWidget(QFrame):
    time_changed = pyqtSignal(int)
    play_state_changed = pyqtSignal(bool)
    
    STEP_MS = 100
    
    def __init__(self, parent=None):
        super().__init__(parent)
        
        self._max_time_ms = 0
        self._current_time_ms = 0
        self._current_cycle = 0
        self._is_playing = False
        self._playback_speed = 1.0
        self._is_live = True
        
        self._playback_timer = QTimer()
        self._playback_timer.timeout.connect(self._on_playback_tick)
        
        self.setMinimumHeight(90)
        self.setMaximumHeight(100)
        self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
        
        self._setup_ui()
        self._apply_styles()
    
    def _setup_ui(self):
        main_layout = QVBoxLayout(self)
        main_layout.setContentsMargins(16, 10, 16, 10)
        main_layout.setSpacing(6)
        
        header_layout = QHBoxLayout()
        header_layout.setSpacing(12)
        
        title = QLabel("Timeline")
        title.setFont(QFont("Segoe UI", 11, QFont.Bold))
        title.setMinimumWidth(60)
        
        self.help_button = HelpButton(
            "<div style='max-width: 450px;'>"
            "<h3 style='margin: 0 0 10px 0; color: #3B82F6;'>Timeline</h3>"
            "<p style='margin: 0 0 10px 0;'>The Timeline allows you to navigate through the simulation history and review past states</p>"
            "<p style='margin: 0 0 5px 0;'><b>Live Mode:</b> Shows real-time data as logs arrive from the simulation</p>"
            "<p style='margin: 0 0 10px 0;'><b>Historical Mode:</b> Activated when you move the slider backward to review past states</p>"
            "<p style='margin: 0 0 5px 0;'><b>|◀ Go to Start:</b> Jump to the beginning of the simulation</p>"
            "<p style='margin: 0 0 5px 0;'><b>◀ Step Back:</b> Go back 100ms in the timeline</p>"
            "<p style='margin: 0 0 5px 0;'><b>▶ Play/Pause:</b> Auto-play through the simulation history</p>"
            "<p style='margin: 0 0 5px 0;'><b>▶ Step Forward:</b> Advance 100ms in the timeline</p>"
            "<p style='margin: 0 0 5px 0;'><b>▶| Go to End:</b> Return to Live mode and see the latest data</p>"
            "</div>",
            icon_size=23
        )
        
        self.mode_label = QLabel("LIVE")
        self.mode_label.setFont(QFont("Segoe UI", 10, QFont.Bold))
        self.mode_label.setStyleSheet("color: #10B981;")
        self.mode_label.setMinimumWidth(80)
        
        speed_label = QLabel("Speed:")
        speed_label.setFont(QFont("Segoe UI", 10))
        
        self.speed_combo = QComboBox()
        self.speed_combo.addItems(["0.25x", "0.5x", "1x", "2x", "4x", "8x"])
        self.speed_combo.setCurrentText("1x")
        self.speed_combo.currentTextChanged.connect(self._on_speed_changed)
        self.speed_combo.setFixedWidth(70)
        
        header_layout.addWidget(title)
        header_layout.addWidget(self.help_button)
        header_layout.addWidget(self.mode_label)
        header_layout.addStretch()
        header_layout.addWidget(speed_label)
        header_layout.addWidget(self.speed_combo)
        
        main_layout.addLayout(header_layout)
        
        controls_layout = QHBoxLayout()
        controls_layout.setSpacing(6)
        
        self.btn_start = IconButton(IconButton.ICON_START)
        self.btn_start.setToolTip("Go to Start")
        self.btn_start.clicked.connect(self._go_to_start)
        
        self.btn_back = IconButton(IconButton.ICON_BACK)
        self.btn_back.setToolTip("Step Back (100ms)")
        self.btn_back.clicked.connect(self._step_back)
        
        self.btn_play = IconButton(IconButton.ICON_PLAY)
        self.btn_play.setToolTip("Play")
        self.btn_play.clicked.connect(self._toggle_play)
        
        self.btn_forward = IconButton(IconButton.ICON_FORWARD)
        self.btn_forward.setToolTip("Step Forward (100ms)")
        self.btn_forward.clicked.connect(self._step_forward)
        
        self.btn_end = IconButton(IconButton.ICON_END)
        self.btn_end.setToolTip("Go to End")
        self.btn_end.clicked.connect(self._go_to_end)
        
        controls_layout.addWidget(self.btn_start)
        controls_layout.addWidget(self.btn_back)
        controls_layout.addWidget(self.btn_play)
        controls_layout.addWidget(self.btn_forward)
        controls_layout.addWidget(self.btn_end)
        
        controls_layout.addSpacing(12)
        
        self.slider = StyledSlider(Qt.Horizontal)
        self.slider.setMinimum(0)
        self.slider.setMaximum(0)
        self.slider.set_format_function(self._format_time_full)
        self.slider.valueChanged.connect(self._on_slider_changed)
        controls_layout.addWidget(self.slider, stretch=1)
        
        controls_layout.addSpacing(12)
        
        self.time_label = QLabel("00:00:00.000 / 00:00:00.000")
        self.time_label.setFont(QFont("Segoe UI", 10))
        self.time_label.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
        self.time_label.setMinimumWidth(180)
        
        self.cycle_label = QLabel("Cycle: 0")
        self.cycle_label.setFont(QFont("Segoe UI", 9))
        self.cycle_label.setProperty("class", "text-secondary")
        self.cycle_label.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
        self.cycle_label.setMinimumWidth(70)
        
        controls_layout.addWidget(self.time_label)
        controls_layout.addWidget(self.cycle_label)
        
        main_layout.addLayout(controls_layout)
    
    def _apply_styles(self):
        bg_primary = current_theme.get('background_primary', '#111827')
        bg_secondary = current_theme.get('background_secondary', '#1F2937')
        bg_tertiary = current_theme.get('background_tertiary', '#4B5563')
        text_primary = current_theme.get('text_primary', '#F9FAFB')
        text_secondary = current_theme.get('text_secondary', '#9CA3AF')
        interactive = current_theme.get('interactive_primary', '#3B82F6')
        
        self.setStyleSheet(f"""
            TimelineWidget {{
                background-color: {bg_secondary};
                border: 1px solid {bg_tertiary};
                border-radius: 10px;
            }}
            QLabel {{
                color: {text_primary};
                background: transparent;
            }}
            QLabel[class="text-secondary"] {{
                color: {text_secondary};
            }}
            QComboBox {{
                background-color: {bg_tertiary};
                border: 1px solid {bg_tertiary};
                border-radius: 4px;
                padding: 4px 8px;
                color: {text_primary};
                font-size: 11px;
            }}
            QComboBox:hover {{
                border-color: {interactive};
            }}
            QComboBox::drop-down {{
                border: none;
            }}
            QComboBox QAbstractItemView {{
                background-color: {bg_secondary};
                border: 1px solid {bg_tertiary};
                selection-background-color: {interactive};
                color: {text_primary};
            }}
        """)
        
        self.slider.apply_style()
    
    def _format_time(self, ms):
        seconds = ms // 1000
        minutes = seconds // 60
        hours = minutes // 60
        
        return f"{hours:02d}:{minutes % 60:02d}:{seconds % 60:02d}"
    
    def _format_time_full(self, ms):
        milliseconds = ms % 1000
        seconds = ms // 1000
        minutes = seconds // 60
        hours = minutes // 60
        
        return f"{hours:02d}:{minutes % 60:02d}:{seconds % 60:02d}.{milliseconds:03d}"
    
    def _update_display(self):
        current_str = self._format_time_full(self._current_time_ms)
        max_str = self._format_time_full(self._max_time_ms)
        
        self.time_label.setText(f"{current_str} / {max_str}")
        self.cycle_label.setText(f"Cycle: {self._current_cycle}")
    
    def set_range(self, max_time_ms):
        self._max_time_ms = max_time_ms
        self.slider.setMaximum(max_time_ms)
        self._update_display()
    
    def set_time(self, time_ms, cycle=None, block_signals=False):
        self._current_time_ms = time_ms
        if cycle is not None:
            self._current_cycle = cycle
        
        if block_signals:
            self.slider.blockSignals(True)
        
        self.slider.setValue(time_ms)
        
        if block_signals:
            self.slider.blockSignals(False)
        
        self._update_display()
    
    def set_cycle(self, cycle):
        self._current_cycle = cycle
        self._update_display()
    
    def get_time(self):
        return self._current_time_ms
    
    def is_playing(self):
        return self._is_playing
    
    def is_live(self):
        return self._is_live
    
    def set_live(self, is_live):
        self._is_live = is_live

    def _go_to_start(self):
        self._is_live = False
        self._stop_playback()
        self.slider.setValue(0)
    
    def _step_back(self):
        self._is_live = False
        new_value = max(0, self._current_time_ms - self.STEP_MS)
        self.slider.setValue(new_value)
    
    def _toggle_play(self):
        if self._is_playing:
            self._stop_playback()
        else:
            self._start_playback()
    
    def _step_forward(self):
        new_value = min(self._max_time_ms, self._current_time_ms + self.STEP_MS)
        
        if new_value >= self._max_time_ms:
            self._is_live = True
        
        self.slider.setValue(new_value)
    
    def _go_to_end(self):
        self._is_live = True
        self._stop_playback()
        self.slider.setValue(self._max_time_ms)
    
    def _start_playback(self):
        self._is_playing = True
        self._is_live = False
        self.btn_play.set_icon_type(IconButton.ICON_PAUSE)
        self.btn_play.setToolTip("Pause")
        
        interval = int(50 / self._playback_speed)
        self._playback_timer.start(max(10, interval))
        
        self.play_state_changed.emit(True)
    
    def _stop_playback(self):
        self._is_playing = False
        self._playback_timer.stop()
        self.btn_play.set_icon_type(IconButton.ICON_PLAY)
        self.btn_play.setToolTip("Play")
        
        self.play_state_changed.emit(False)
    
    def _on_playback_tick(self):
        step = int(50 * self._playback_speed)
        new_value = self._current_time_ms + step
        
        if new_value >= self._max_time_ms:
            new_value = self._max_time_ms
            self._stop_playback()
            self._is_live = True
        
        self.slider.setValue(new_value)
    
    def _on_speed_changed(self, speed_text):
        speed_map = {
            "0.25x": 0.25,
            "0.5x": 0.5,
            "1x": 1.0,
            "2x": 2.0,
            "4x": 4.0,
            "8x": 8.0
        }
        self._playback_speed = speed_map.get(speed_text, 1.0)
        
        if self._is_playing:
            self._playback_timer.stop()
            interval = int(50 / self._playback_speed)
            self._playback_timer.start(max(10, interval))
    
    def _on_slider_changed(self, value):
        self._current_time_ms = value
        self._update_display()
        
        self.time_changed.emit(value)
    
    def update_theme(self):
        self._apply_styles()
        self.update()


import copy
import re 
import ast 
from PyQt5.QtWidgets import (QWidget, QVBoxLayout, QHBoxLayout, QLabel, 
                             QScrollArea, QFrame, QTreeWidget, QTreeWidgetItem, 
                             QHeaderView, QTextEdit, QPushButton, QButtonGroup,
                             QListWidget, QListWidgetItem, QSizePolicy)
from PyQt5.QtCore import Qt
from PyQt5.QtGui import QFont
from gui.assets.theme.styler import current_theme
from gui.widgets.percepts_table import PerceptsTableWidget
from gui.widgets.help_button import HelpButton


class AmbientePage(QWidget):
    def __init__(self, log_store):
        super().__init__()
        self.setProperty("class", "page")
        self.log_store = log_store
        
        self.known_environments = set()
        self.selected_environment = None
        self.env_button_group = QButtonGroup(self)
        self.env_button_group.setExclusive(True)

        self._setup_ui()

        self.log_store.environment_state_updated.connect(self.on_environment_state_updated)
        self.log_store.environment_history_updated.connect(self.on_environment_history_updated)
        self.log_store.store_updated.connect(self.check_timeline_update)

        self.on_environment_state_updated(self.log_store.get_environment_states_at_index(0))

    def _setup_ui(self):
        main_layout = QHBoxLayout(self)
        main_layout.setContentsMargins(20, 20, 20, 20)
        main_layout.setSpacing(15)

        left_column = QFrame()
        left_column.setFrameShape(QFrame.StyledPanel)
        left_column.setProperty("class", "card")
        left_column.setMaximumWidth(250)
        left_layout = QVBoxLayout(left_column)

        left_title_layout = QHBoxLayout()
        left_title = QLabel("Environments")
        left_title.setProperty("class", "h2")
        left_title.setFont(QFont("Segoe UI", 14, QFont.Bold))
        
        self.help_button = HelpButton(
            "<div style='max-width: 450px;'>"
            "<h3 style='margin: 0 0 10px 0; color: #3B82F6;'>Environments</h3>"
            "<p style='margin: 0 0 10px 0;'>Environments are spaces where the agents are situated.</p>"
            "<p style='margin: 0 0 5px 0;'><b>Connected Agents:</b> List of agents currently linked to this environment</p>"
            "<p style='margin: 0 0 5px 0;'><b>Percepts:</b> Information available in the environment that agents can perceive and use to update their beliefs</p>"
            "<p style='margin: 0 0 10px 0;'><b>Changes History:</b> Chronological log of all modifications in the environment</p>"
            "</div>",
            icon_size=28
        )
        
        left_title_layout.addWidget(left_title)
        left_title_layout.addStretch()
        left_title_layout.addWidget(self.help_button)
        left_layout.addLayout(left_title_layout)
        
        self.env_scroll_area = QScrollArea()
        self.env_scroll_area.setWidgetResizable(True)
        self.env_scroll_area.setFrameShape(QFrame.NoFrame)
        
        self.env_list_widget = QWidget()
        self.env_list_layout = QVBoxLayout(self.env_list_widget)
        self.env_list_layout.setAlignment(Qt.AlignTop)
        self.env_list_layout.setSpacing(4)
        self.env_scroll_area.setWidget(self.env_list_widget)
        
        left_layout.addWidget(self.env_scroll_area)
        main_layout.addWidget(left_column)

        self.details_container = QWidget()
        self.details_layout = QVBoxLayout(self.details_container)
        self.details_layout.setContentsMargins(0, 0, 0, 0)
        main_layout.addWidget(self.details_container, stretch=1)

        self._show_placeholder()

    def update_theme(self):
        if self.selected_environment:
            self.rebuild_history_log()
            if hasattr(self, 'percepts_table'):
                self.percepts_table.update_theme()

    def _show_placeholder(self):
        self._clear_layout(self.details_layout)
        placeholder = QLabel("Select an environment to view details")
        placeholder.setAlignment(Qt.AlignCenter)
        placeholder.setProperty("class", "text-secondary")
        placeholder.setFont(QFont("Segoe UI", 12))
        self.details_layout.addWidget(placeholder)

    def _clear_layout(self, layout):
        while layout.count():
            child = layout.takeAt(0)
            if child.widget():
                child.widget().deleteLater()
                
    def _build_details_ui(self, env_name):
        self._clear_layout(self.details_layout)

        title = QLabel(f"Monitoring: {env_name}")
        title.setProperty("class", "h1")
        title.setFont(QFont("Segoe UI", 18, QFont.Bold))
        self.details_layout.addWidget(title)

        top_layout = QHBoxLayout()
        top_layout.setSpacing(15)

        agents_frame = QFrame()
        agents_frame.setProperty("class", "card")
        agents_layout = QVBoxLayout(agents_frame)
        agents_layout.setContentsMargins(16, 16, 16, 16)
        
        agents_title = QLabel("Connected Agents")
        agents_title.setProperty("class", "h2")
        agents_title.setFont(QFont("Segoe UI", 12, QFont.Bold))
        
        self.agents_list = QListWidget()
        self.agents_list.setFrameShape(QFrame.NoFrame)
        self._style_list(self.agents_list)
        
        agents_layout.addWidget(agents_title)
        agents_layout.addWidget(self.agents_list)
        top_layout.addWidget(agents_frame, stretch=1)

        percepts_frame = QFrame()
        percepts_frame.setProperty("class", "card")
        percepts_layout = QVBoxLayout(percepts_frame)
        percepts_layout.setContentsMargins(16, 16, 16, 16)

        percepts_header = QHBoxLayout()
        percepts_title = QLabel("Percepts")
        percepts_title.setProperty("class", "h2")
        percepts_title.setFont(QFont("Segoe UI", 12, QFont.Bold))
        
        self.percepts_counter = QLabel("(0)")
        self.percepts_counter.setProperty("class", "text-secondary")
        self.percepts_counter.setFont(QFont("Segoe UI", 11))
        
        percepts_header.addWidget(percepts_title)
        percepts_header.addWidget(self.percepts_counter)
        percepts_header.addStretch()
        
        percepts_layout.addLayout(percepts_header)
        
        percepts_scroll = QScrollArea()
        percepts_scroll.setWidgetResizable(True)
        percepts_scroll.setFrameShape(QFrame.NoFrame)
        percepts_scroll.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        
        self.percepts_table = PerceptsTableWidget()
        percepts_scroll.setWidget(self.percepts_table)
        
        percepts_layout.addWidget(percepts_scroll)
        top_layout.addWidget(percepts_frame, stretch=2)

        self.details_layout.addLayout(top_layout)

        history_frame = QFrame()
        history_frame.setProperty("class", "card")
        history_layout = QVBoxLayout(history_frame)
        history_layout.setContentsMargins(16, 16, 16, 16)
        
        history_title = QLabel("Changes History")
        history_title.setProperty("class", "h2")
        history_title.setFont(QFont("Segoe UI", 12, QFont.Bold))
        
        self.history_log = QTextEdit()
        self.history_log.setReadOnly(True)
        self.history_log.setMinimumHeight(200)
        self.history_log.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        self.history_log.setFrameShape(QFrame.NoFrame)
        
        history_layout.addWidget(history_title)
        history_layout.addWidget(self.history_log)
        
        self.details_layout.addWidget(history_frame)
        self.details_layout.setStretchFactor(history_frame, 1)

    def _style_list(self, list_widget):
        bg_secondary = current_theme.get('background_secondary', '#1F2937')
        bg_tertiary = current_theme.get('background_tertiary', '#4B5563')
        text_primary = current_theme.get('text_primary', '#F9FAFB')
        interactive = current_theme.get('interactive_primary', '#3B82F6')
        
        list_widget.setStyleSheet(f"""
            QListWidget {{
                background-color: transparent;
                border: none;
                outline: none;
            }}
            QListWidget::item {{
                background-color: {bg_secondary};
                border: 1px solid {bg_tertiary};
                border-radius: 6px;
                padding: 10px 12px;
                margin: 2px 0px;
                color: {text_primary};
            }}
            QListWidget::item:selected {{
                background-color: {interactive};
                border: 1px solid {interactive};
            }}
        """)

    def _add_environment_button(self, env_name):
        button = QPushButton(env_name)
        button.setObjectName("FilterButton")
        button.setCheckable(True)
        button.setCursor(Qt.PointingHandCursor)
        button.clicked.connect(lambda checked, n=env_name: self.on_environment_selected(n))
        self.env_button_group.addButton(button)
        
        for i in range(self.env_list_layout.count()):
            widget = self.env_list_layout.itemAt(i).widget()
            if widget and widget.text() > env_name:
                self.env_list_layout.insertWidget(i, button)
                return
        self.env_list_layout.addWidget(button)

    def on_environment_selected(self, env_name):
        if self.selected_environment == env_name:
            return
            
        self.selected_environment = env_name
        self._build_details_ui(env_name)

        idx = self.log_store.current_timeline_index
        all_states = self.log_store.get_environment_states_at_index(idx)
        
        self.on_environment_state_updated(all_states)
        self.rebuild_history_log()

    def check_timeline_update(self):
        if self.isVisible() and self.selected_environment:
            self.on_timeline_state_changed(self.log_store.current_timeline_index)

    def on_environment_state_updated(self, all_envs_data):
        if not all_envs_data:
            return

        for env_name in all_envs_data.keys():
            if env_name not in self.known_environments:
                self.known_environments.add(env_name)
                self._add_environment_button(env_name)

        if self.selected_environment and self.selected_environment in all_envs_data:
            data = all_envs_data[self.selected_environment]
            self.agents_list.clear()
            agents = data.get('connected_agents', [])
            if not agents:
                item = QListWidgetItem("No agents connected")
                item.setForeground(Qt.gray)
                self.agents_list.addItem(item)
            else:
                for agent in sorted(agents):
                    self.agents_list.addItem(QListWidgetItem(agent))

            percepts = data.get('percepts', {})
            self.percepts_table.set_percepts(percepts)

            total_percepts = len(self.percepts_table._all_percepts)
            self.percepts_counter.setText(f"({total_percepts})")

    def on_environment_history_updated(self, env_name, history_entry):
        if env_name == self.selected_environment and self.log_store.is_live:
            self._format_and_add_history_entry(history_entry)
            sb = self.history_log.verticalScrollBar()
            sb.setValue(sb.maximum())

    def rebuild_history_log(self):
        if not self.selected_environment:
            return
            
        self.history_log.clear()
        history_list = self.log_store.get_environment_change_history(self.selected_environment)

        for entry in history_list:
            self._format_and_add_history_entry(entry)

    def _format_and_add_history_entry(self, entry):
        raw_type = getattr(entry, 'type', 'unknown') or 'unknown'
        content = str(getattr(entry, 'content', 'N/A')).replace('<', '&lt;').replace('>', '&gt;')
        action = str(getattr(entry, 'agent_action', 'N/A'))
        time = getattr(entry, 'system_time', 'N/A')

        log_type_key = raw_type.lower()

        color_map = {
            'create': current_theme.get('success', '#10B981'),
            'delete': current_theme.get('danger', '#EF4444'),
            'change': current_theme.get('warning', '#F59E0B'),
            'update': current_theme.get('info', '#3B82F6')
        }
        
        type_map = {
            'create': '[CREATE]',
            'delete': '[DELETE]',
            'change': '[CHANGE]',
            'update': '[UPDATE]'
        }

        color = color_map.get(log_type_key, current_theme.get('text_primary', '#FFFFFF'))
        type_str = type_map.get(log_type_key, f'[{raw_type.upper()}]')

        text_sec_color = current_theme.get('text_secondary', '#9CA3AF')
        text_pri_color = current_theme.get('text_primary', '#FFFFFF')

        html = (
            f'<span style="color:{text_sec_color};">[{time}]</span> '
            f'<span style="color:{color}; font-weight:bold;">{type_str}</span> '
            f'<span style="color:{text_pri_color};">{content}</span> '
            f'<span style="color:{text_sec_color}; font-style:italic;"> (Action: {action})</span>'
        )
        self.history_log.append(html)

    def on_timeline_state_changed(self, index):
        states_at_index = self.log_store.get_environment_states_at_index(index)
        self.on_environment_state_updated(states_at_index)
        
        if self.selected_environment:
            self.rebuild_history_log()


from PyQt5.QtCore import QThread, pyqtSignal, Qt
from PyQt5.QtWidgets import (QWidget, QVBoxLayout, QHBoxLayout, 
                             QStackedWidget, QLabel, QFrame,
                             QApplication) 
from gui.widgets.side_bar import SideBar
from gui.widgets.top_bar import TopBar
from gui.widgets.timeline_widget import TimelineWidget
from gui.views.page_home import MenuInicialPage
from gui.views.page_agents import AgentesPage
from gui.views.page_messages import MensagensPage
from gui.views.page_environment import AmbientePage
from gui.views.page_learning import PageLearning
from gui.core.log_store import LogStore
from gui.core.data_model import AgentDataModel

from gui.widgets.settings_dialog import SettingsDialog
from gui.widgets.manual_dialog import ManualDialog
from gui.assets.theme.styler import load_stylesheet, set_active_theme, current_theme

class InterfaceWindow(QWidget):
    
    timeline_index_changed = pyqtSignal(int)

    def __init__(self, command_queue, num_agents, new_queue):
        super().__init__()
        self.setWindowTitle("MASPY GUI")
        self.setGeometry(100, 100, 1200, 700)
        
        self._current_theme = "dark" 

        self.command_queue = command_queue

        self.log_thread = QThread()
        self.log_store = LogStore(new_queue)
        self.log_store.moveToThread(self.log_thread)
        self.log_thread.started.connect(self.log_store.start_polling)
        self.log_thread.finished.connect(self.log_thread.deleteLater)

        self.data_model = AgentDataModel(self.log_store)

        self._setup_ui()

        self.timeline_index_changed.connect(self.log_store.set_current_timeline_index)
        
        self.timeline_index_changed.connect(self.page_agentes.on_store_updated)
        self.timeline_index_changed.connect(self.page_mensagens.on_store_updated)
        self.timeline_index_changed.connect(self.page_ambiente.on_timeline_state_changed)
        self.timeline_index_changed.connect(self.page_menu.on_store_updated)
        self.timeline_index_changed.connect(self.data_model.on_store_updated)

        self.log_thread.start()
        
        self.settings_dialog = None
        self.manual_dialog = None

    def _setup_ui(self):
        main_layout = QHBoxLayout(self)
        main_layout.setContentsMargins(0, 0, 0, 0)
        main_layout.setSpacing(0)

        self.sidebar = SideBar()
        main_layout.addWidget(self.sidebar)

        right_layout = QVBoxLayout()
        right_layout.setContentsMargins(0, 0, 0, 0)
        right_layout.setSpacing(0)

        self.topbar = TopBar("Menu")
        self.topbar.settings_button_clicked.connect(self._open_settings)
        self.topbar.manual_button_clicked.connect(self._open_manual)
        right_layout.addWidget(self.topbar)
        
        self.timeline = TimelineWidget()
        right_layout.addWidget(self.timeline)

        self.stack = QStackedWidget()
        
        self.page_menu = MenuInicialPage(self.command_queue, self.log_store)
        self.page_agentes = AgentesPage(self.log_store, self.data_model)
        self.page_mensagens = MensagensPage(self.log_store)
        
        self.page_ambiente = AmbientePage(self.log_store)
        self.page_ambiente.setObjectName("page_ambiente") 
        
        self.page_learning = PageLearning(self.log_store)

        self.stack.addWidget(self.page_menu)
        self.stack.addWidget(self.page_agentes)
        self.stack.addWidget(self.page_ambiente)
        self.stack.addWidget(self.page_mensagens)
        self.stack.addWidget(self.page_learning)
        
        right_layout.addWidget(self.stack)
        main_layout.addLayout(right_layout)

        self.log_store.agent_list_updated.connect(self.page_agentes.on_agent_list_updated)
        
        self.log_store.store_updated.connect(self.page_agentes.on_store_updated)
        self.log_store.store_updated.connect(self.page_mensagens.on_store_updated)
        self.log_store.store_updated.connect(self.page_menu.on_store_updated)

        self.log_store.store_updated.connect(self._update_timeline_range)
        self.timeline.time_changed.connect(self._on_timeline_time_changed)
        
        self.sidebar.buttons["Menu"].clicked.connect(lambda: self.show_page(0, "Menu"))
        self.sidebar.buttons["Agents"].clicked.connect(lambda: self.show_page(1, "Agents"))
        self.sidebar.buttons["Environment"].clicked.connect(lambda: self.show_page(2, "Environment"))
        self.sidebar.buttons["Messages"].clicked.connect(lambda: self.show_page(3, "Messages"))
        self.sidebar.buttons["Learning"].clicked.connect(lambda: self.show_page(4, "Learning"))

        self.sidebar.buttons["Menu"].setChecked(True)

    def _open_settings(self):
        if self.settings_dialog is None:
            self.settings_dialog = SettingsDialog(self)
            self.settings_dialog.theme_changed.connect(self._apply_theme)
            self.settings_dialog.setAttribute(Qt.WA_DeleteOnClose)
            self.settings_dialog.destroyed.connect(self._on_settings_closed)

        if self._current_theme == 'light':
            self.settings_dialog.btn_light.setChecked(True)
        else:
            self.settings_dialog.btn_dark.setChecked(True)
            
        self.settings_dialog.show()
        self.settings_dialog.activateWindow()

    def _on_settings_closed(self):
        self.settings_dialog = None

    def _open_manual(self):
        if self.manual_dialog is None:
            self.manual_dialog = ManualDialog(self)
            self.manual_dialog.setAttribute(Qt.WA_DeleteOnClose)
            self.manual_dialog.destroyed.connect(self._on_manual_closed)
            
        self.manual_dialog.show()
        self.manual_dialog.activateWindow()

    def _on_manual_closed(self):
        self.manual_dialog = None

    def _apply_theme(self, theme_name):
        if theme_name == self._current_theme:
            return 

        set_active_theme(theme_name)
        self._current_theme = theme_name 

        app_instance = QApplication.instance()
        if app_instance:
            stylesheet = load_stylesheet(theme_name)
            app_instance.setStyleSheet(stylesheet)
        self._propagate_theme_to_pages()

    def _propagate_theme_to_pages(self):
        pages = [
            self.page_menu,
            self.page_agentes,
            self.page_mensagens,
            self.page_ambiente,
            self.page_learning
        ]
        
        for page in pages:
            if hasattr(page, 'update_theme'):
                try:
                    page.update_theme()
                except Exception as e:
                    print(f"Erro ao atualizar tema em {page}: {e}")
        
        if hasattr(self, 'timeline'):
            self.timeline.update_theme()

    def _update_timeline_range(self):
        new_max_ms = self.log_store.get_total_duration_ms()
        if new_max_ms == 0: 
            return

        self.timeline.set_range(new_max_ms)
        
        if self.log_store.is_live:
            cycle = self.log_store.get_message_count_limit()
            self.timeline.set_time(new_max_ms, cycle, block_signals=True)
            self.timeline.set_live(True)

    def _on_timeline_time_changed(self, ms_value):
        max_val_ms = self.log_store.get_total_duration_ms()
        
        index = self.log_store.get_index_from_ms(ms_value)
        self.log_store.set_current_timeline_index(index)
        
        is_live = (ms_value >= max_val_ms - 100)
        self.log_store.is_live = is_live
        self.timeline.set_live(is_live)

        self.timeline.set_cycle(index)

        self.timeline_index_changed.emit(index)

    def show_page(self, index, title):
        self.stack.setCurrentIndex(index)
        self.topbar.set_title(title)

    def closeEvent(self, event):
        self.log_thread.quit()
        self.log_thread.wait()
        super().closeEvent(event)